/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package euler.utilities;

import euler.DiagramPanel;


/**
 * As a GraphView, but for DiagramPanel
 *
 */


public abstract class DiagramUtility {

	/** the associated DiagramPanel */
	protected DiagramPanel diagramPanel = null;
	/** the key that should be pressed to start the graph drawing method. Codes can be found in the KeyEvent class. */
	protected int acceleratorKey = 0;
	/** the message that should appear in menu options to start the graph drawing method */
	protected String menuText;
	/** the menu mnemonic for initiating the layout. Codes can be found in the KeyEvent class. */
	protected int mnemonicKey = 0;

	/** Minimal constructor. */
	public DiagramUtility(String inMenuText) {
		menuText = inMenuText;
	}

	/** Trivial constructor. */
	public DiagramUtility(int inAccelerator, String inMenuText) {
		acceleratorKey = inAccelerator;
		menuText = inMenuText;
	}

	/** Trivial constructor. */
	public DiagramUtility(int inAccelerator, String inMenuText, int inMnemonic) {
		acceleratorKey = inAccelerator;
		menuText = inMenuText;
		mnemonicKey = inMnemonic;
	}

	/** Trival accessor. */
	public DiagramPanel getDiagramPanel() {return diagramPanel;}
	/** Trival accessor. */
	public int getAcceleratorKey() {return acceleratorKey;}
	/** Trival accessor. */
	public String getMenuText() {return menuText;}
	/** Trival accessor. */
	public int getMnemonicKey() {return mnemonicKey;}

	/** This modifier should only be used by DiagramPanel. */
	public void setDiagramPanel(DiagramPanel inDiagramPanel) {diagramPanel=inDiagramPanel;}
	/** Trivial modifier. */
	public void setAcceleratorKey(int inKey) {acceleratorKey=inKey;}
	/** Trivial modifier. */
	public void setMenuText(String inMenuText) {menuText=inMenuText;}
	/** Trivial modifier. */
	public void setMnemonicKey(int inKey) {mnemonicKey=inKey;}

	/** Overwrite this with the modification to the graph. */
	public abstract void apply();


}



