/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;


/**
 * To instantiate and handle a Line as defined in eulerForce
 * 
 */


public class MyLine {
	
	private double x1;
	private double y1;
	private double x2;
	private double y2;
	
	public MyLine (double in_x1, double in_y1, double in_x2, double in_y2){
		x1 = in_x1;
		y1 = in_y1;
		x2 = in_x2;
		y2 = in_y2;
	}
	public MyLine (Point2D.Double inPnt1, Point2D.Double inPnt2){
		x1 = inPnt1.x;
		y1 = inPnt1.y;
		x2 = inPnt2.x;
		y2 = inPnt2.y;
	}
	public MyLine (Line2D.Double line){
		x1 = line.x1;
		y1 = line.y1;
		x2 = line.x2;
		y2 = line.y2;
	}
	
	
	
	public Point2D.Double getxyAbsDistances (){
		Point2D.Double xyAbsDistances = getxySignedDistances();
		xyAbsDistances.x = Math.abs(xyAbsDistances.x);
		xyAbsDistances.y = Math.abs(xyAbsDistances.y);
		return xyAbsDistances;
	}
	public Point2D.Double getxySignedDistances (){
		Point2D.Double xyDistances = new Point2D.Double();
		xyDistances.x = x2-x1;
		xyDistances.y = y2-y1;
		return xyDistances;
	}
	public Double getLineLengthSqrd (){
		Point2D.Double xyAbsDistances = getxyAbsDistances();
		return (Math.pow(xyAbsDistances.x, 2)+ Math.pow(xyAbsDistances.y, 2));
	}
	public Double getLineLength (){
		return (Math.sqrt(getLineLengthSqrd()));
	}

	
	public double getVectorPntsDeterminant(){
		return ((x1*y2)-(x2*y1));
	}
	
	
	public double getGradient(){
		return ((y2-y1)/(x2-x1));
	}
	public double getYIntercept(){
		return (y1 - (getGradient()*x1));
	}
	
	public double getYKnowingX (double x){
		return ((this.getGradient()*x)+this.getYIntercept());
	}
	public double getXKnowingY (double y){
		return ((y-this.getYIntercept())/this.getGradient());
	}
	
	public boolean isPointOnLine(Point2D.Double pnt){
		return (pnt.y == getYKnowingX(pnt.x));
	}
	public boolean isPointOnLineSeg(Point2D.Double pnt) {
		return isPointOnLineSeg(pnt, new Point2D.Double(0.0,0.0));
	}
	public boolean isPointOnLineSeg(Point2D.Double pnt, Point2D.Double xyAllowance){
		if (!isPointOnLine(pnt)){return false;}
		double xbound1 = this.x1;
		double xbound2 = this.x2;
		double ybound1 = this.y1;
		double ybound2 = this.y2;		
		if (this.x1 > this.x2){
			xbound1 = this.x2;
			xbound2 = this.x1;
		}
		if (this.y1 > this.y2){
			ybound1 = this.y2;
			ybound2 = this.y1;
		}
		xbound1 -= xyAllowance.x;
		xbound2 += xyAllowance.x;
		ybound1 -= xyAllowance.y;
		ybound2 += xyAllowance.y;	
		
		return ((pnt.x >= xbound1) && (pnt.x <= xbound2) && (pnt.y >= ybound1) && (pnt.y <= ybound2));
	}
	
	public boolean doesLineIntersectLine(MyLine lineSeg2){
		return (this.getGradient() != lineSeg2.getGradient());
	}
	public Point2D.Double getLineIntersectionPointWithLine(MyLine lineSeg2){
	// here consider actual lines (represented by an equation) NOT line segments	
		
		if (!doesLineIntersectLine(lineSeg2)){return null;}
		
		Point2D.Double intPnt = new Point2D.Double(0,0);
		double l1Grad = this.getGradient();
		double l1YInt = this.getYIntercept();
		double l2Grad = lineSeg2.getGradient();
		double l2YInt = lineSeg2.getYIntercept();		
		
		// equation: x = (c2-c1)/(m1-m2) , y -> substitute in one of the line equations
		intPnt.x = (l2YInt-l1YInt) / (l1Grad-l2Grad);
		intPnt.y = getYKnowingX(intPnt.x);
	
		return intPnt;
	}
	public Point2D.Double getLineIntersectionPointWithLineSeg(MyLine lineSeg2){
		Point2D.Double intPnt = getLineIntersectionPointWithLine(lineSeg2);
		if (intPnt == null){return null;}
		if (lineSeg2.isPointOnLineSeg(intPnt)){
			return intPnt;
		}else{
			return null;
		}
	}
	public Point2D.Double getLineSegIntersectionPointWithLine(MyLine lineSeg2){
		Point2D.Double intPnt = getLineIntersectionPointWithLine(lineSeg2);
		if (intPnt == null){return null;}
		if (this.isPointOnLineSeg(intPnt)){
			return intPnt;
		}else{
			return null;
		}
	}
	public Point2D.Double getLineSegIntersectionPointWithLineSeg(MyLine lineSeg2){
		Point2D.Double intPnt = getLineIntersectionPointWithLine(lineSeg2);
		if (intPnt == null){return null;}
		if (this.isPointOnLineSeg(intPnt)&&lineSeg2.isPointOnLineSeg(intPnt)){
			return intPnt;
		}else{
			return null;
		}
	}
	
	public double getPerpLineGradient(){
		return (-1/getGradient());
	}
	public double getPerpLineYIntercept(Point2D.Double pntOnPerpLine){
		return (pntOnPerpLine.y - (getPerpLineGradient()*pntOnPerpLine.x));
	}
	public Point2D.Double getIntPntWithPerpLine(Point2D.Double pntOnPerpLine){
		double lineGrad = getGradient();
		double lineYInt = getYIntercept();
		
		double perpLineGrad = getPerpLineGradient();
		double perpLineYInt = getPerpLineYIntercept(pntOnPerpLine);
		
		double x = (perpLineYInt - lineYInt) / (lineGrad - perpLineGrad);
		double y = getYKnowingX(x); //(lineGrad * x) + lineYInt; 
		
		return (new Point2D.Double(x,y));
	}
	
}
