/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;

import java.awt.Polygon;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;

import euler.ConcreteContour;
import euler.ConcreteDiagram;
import euler.ContourLine;


/**
 * To instantiate, define and handle Euler diagrams whose curves are polygons in eulerForce
 * 
 */


public class SimpleConcreteDiagram {
	
	// Public Class Values 
 	public static final String FILESTARTABSTRACTDESCRIPTION  = "ABSTRACTDESCRIPTION";
 	public static final String FILESTARTDIAGRAM = "DIAGRAM";
 	public static final String FILESTARTCONTOURS = "CONTOURS";
 	public static final char FILESEPARATOR = '|';
 	
 	
	// Object Data Members
	protected ArrayList <ConcreteContour> concreteContours;
	
	
	// Constructors 
	
	public SimpleConcreteDiagram (){
		setConcreteContours(new ArrayList<ConcreteContour>());
	}
	public SimpleConcreteDiagram (ConcreteDiagram inConcreteDiagram){
		setConcreteContours(inConcreteDiagram.getConcreteContours());
	}
	
	
	// Properties
	
	public ArrayList <ConcreteContour> getConcreteContours(){
		return concreteContours;
	}
	@SuppressWarnings("unchecked")
	public void setConcreteContours(ArrayList <ConcreteContour> inConcreteContours){
		concreteContours = (ArrayList <ConcreteContour>) inConcreteContours.clone();
	}
	
	
	// Methods 
	
	
	public SimpleConcreteDiagram clone(){
		
		SimpleConcreteDiagram scd_clone = new SimpleConcreteDiagram();
		
		ArrayList<ConcreteContour> concreteContoursClone = new ArrayList<ConcreteContour>(concreteContours.size());  
		for (ConcreteContour c : concreteContours){
			concreteContoursClone.add(c.clone());
		}
		scd_clone.setConcreteContours(concreteContoursClone);
		
		return scd_clone;
	}
	
	
	
	public void clear() {
		concreteContours = new ArrayList<ConcreteContour>();
	}
	
	public static String getAbstractDescriptionFromFile (File file){
		
		String abstractDescription = ""; 
		
		try {
			BufferedReader b = new BufferedReader(new FileReader(file));			
		
			boolean readingAbstractDescription = false;		
			boolean found = false;
			String line = b.readLine();			
			while((line != null) && (!found)) { 				
				if(line.equals("")) {
					line = b.readLine();
					continue;
				}
				if(readingAbstractDescription && line.compareTo(FILESTARTABSTRACTDESCRIPTION)!=0 && line.compareTo(FILESTARTCONTOURS)!=0) {
					abstractDescription = line;
					found = true;
				}
				if(line.compareTo(FILESTARTABSTRACTDESCRIPTION)==0) {
					readingAbstractDescription = true;				
				}						
				line = b.readLine();				
			}			
			b.close();
		} catch(IOException e){
			System.out.println("An IO exception occurred when executing getAbstractDescriptionFromFile("+file+") in SimpleConcreteDiagram.java: "+e+"\n");
			System.exit(1);
		}
		return abstractDescription;
	}
	
	public boolean loadDiagram(File file){
		try{
			BufferedReader b = new BufferedReader(new FileReader(file));
			return loadDiagram(b);
		} catch(IOException e){
			return false;
		}
		
	}
	
	public boolean loadDiagram(BufferedReader b){
		
		try {		
			Character c = new Character(FILESEPARATOR);
			String separatorString = new String(c.toString());
			
			concreteContours = new ArrayList<ConcreteContour>();
			String abstractDescription = ""; // the abstract description is retrieved from the file but then it's never used   
			
			boolean readingAbstractDescription = false;
			boolean readingContours = false;			
			String line = b.readLine();			
			while(line != null) { 				
				if(line.equals("")) {
					line = b.readLine();
					continue;
				}
				if(readingAbstractDescription && line.compareTo(FILESTARTABSTRACTDESCRIPTION)!=0 && line.compareTo(FILESTARTCONTOURS)!=0) {
					abstractDescription = line;
				}
				if(readingContours && line.compareTo(FILESTARTCONTOURS)!=0){ 
					StringBuffer parseLine = new StringBuffer(line);
					int separatorInd = 0;
					
					// get contour label
					separatorInd = parseLine.indexOf(separatorString);
					String label = parseLine.substring(0,separatorInd);
					parseLine.delete(0,separatorInd+1);
					
					//get coordinates					
					ArrayList<String> XYCor = new ArrayList<String>();
					
					while(parseLine.length()!=0){
						separatorInd = parseLine.indexOf(separatorString);
						String cor = parseLine.substring(0,separatorInd);
						XYCor.add(cor);
						parseLine.delete(0,separatorInd+1);
					}
					int xy[] = new int[XYCor.size()];					
					for(int i = 0 ; i < XYCor.size(); i++){
						xy[i] = new Integer(Integer.parseInt(XYCor.get(i)));					
					}					
					Polygon pol = new Polygon();
					for(int i = 0 ; i < xy.length; i+=2){
						pol.addPoint(xy[i],xy[i+1]);						
					}					
					ConcreteContour cc = new ConcreteContour(label, pol);					
					concreteContours.add(cc);
				}				
				if(line.compareTo(FILESTARTABSTRACTDESCRIPTION)==0) {
					readingAbstractDescription = true;
					readingContours = false;				
				}				
				if(line.compareTo(FILESTARTCONTOURS)==0) {
					readingContours = true;
				}		
				line = b.readLine();				
			}			
			b.close();
		} catch(IOException e){
			System.out.println("An IO exception occurred when executing loadAdjacencyFile() in SimpleConcreteDiagram.java: "+e+"\n");
			System.exit(1);
		}
		
		if (concreteContours.size() > 0){
			return(true);
		} else {
			return(false);
		}
	}
	
	
	public boolean saveToFile(File file) {	
		
		try {
			BufferedWriter b = new BufferedWriter(new FileWriter(file));
			
			b.append(FILESTARTDIAGRAM);
			b.newLine(); 
			b.newLine();
			b.append(FILESTARTABSTRACTDESCRIPTION);
			b.newLine();
			b.append(ConcreteContour.generateAbstractDiagramFromList(concreteContours));
			b.newLine();
			b.newLine();
			b.append(FILESTARTCONTOURS);
			b.newLine();
				
			for(ConcreteContour cc : concreteContours){
				StringBuffer outputContour = new StringBuffer("");
				outputContour.append(cc.getAbstractContour());
				outputContour.append(FILESEPARATOR);
				Polygon pol = cc.getPolygon();
				for(int i = 0 ; i < pol.npoints; i++){
					outputContour.append(pol.xpoints[i]);
					outputContour.append(FILESEPARATOR);
					outputContour.append(pol.ypoints[i]);
					outputContour.append(FILESEPARATOR);
				}
				b.append(outputContour);
				b.newLine();
			}		
			b.close();
		}
		catch(IOException e) {
			System.out.println("An IO exception occurred when executing saveAll() in ThreeSetDiagramLibrary.java "+e+"\n");
			return false;
		}
		return true;
	}
	
	
	public void printContourPointDetails(){
		System.out.println("--- Contour Points Details for Diagram " + ConcreteContour.generateAbstractDiagramFromList(concreteContours)+ " ---");
		for (ConcreteContour cc : concreteContours){
			
			cc.setContourLines();
			ArrayList<ContourLine> cls = cc.getContourLines();
			SimpleDiagramDrawerForceModel fm = new SimpleDiagramDrawerForceModel();
			for (ContourLine cl : cls){
				Point2D.Double clpnt1 = new Point2D.Double (cl.getLine().x1, cl.getLine().y1);
				Point2D.Double clpnt2 = new Point2D.Double (cl.getLine().x2, cl.getLine().y2);
				Point2D.Double xyDist = fm.getxyAbsDistances(clpnt1, clpnt2);
				double dist = fm.getAbsHypotenuse(xyDist);
				
				System.out.println(" - ("+clpnt1.x+","+clpnt1.y+") - ("+clpnt2.x+","+clpnt2.y+"): x="+xyDist.x+", y="+xyDist.y+", dist="+dist);
			}
		}
		System.out.println("-----------------------------------------------------------------------------------------------------");
	}
	
	
	
	
	public static File[] getDiagFilesInDir (File dir){
		FilenameFilter filterDiagDescFiles = new FilenameFilter() {
		public boolean accept(File dir, String name) {
			return name.endsWith(".edp");
		}};
			
		return dir.listFiles(filterDiagDescFiles);
	}
		
}
