/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.event.KeyEvent;
import java.awt.geom.Area;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;


import euler.ConcreteContour;
import euler.ContourLine;
import euler.ConcreteDiagram;


/**
 * To instantiate and define the force model and algorithm of eulerForce's force-directed approach to lay out Euler diagrams 
 * 
 */


public class SimpleDiagramDrawerForceModel extends SimpleDiagramDrawer{
	

	//Class Default Values 
	public static final String DEFAULT_MENUTEXT = "Force-Directed";
	private static final int DEFAULT_ACCELERATORKEY = KeyEvent.VK_F; 
	
	protected static final int DEFAULT_I = 600;  					
	protected static final int DEFAULT_I_for3 = 200;
	protected static final int DEFAULT_I_for4 =220;
	protected static final int DEFAULT_I_for5 = 350;
	
	protected static final double DEFAULT_A = .5;
	protected static final double DEFAULT_A_for3 = .5;
	protected static final double DEFAULT_A_for4 = .95;
	protected static final double DEFAULT_A_for5= .8;
	
	protected static final double DEFAULT_A_CC = 0;
	protected static final double DEFAULT_AD = .15;
	protected static final double DEFAULT_ADb = .1;
	protected static final double DEFAULT_AD2 = .2; 
	protected static final double DEFAULT_AD3 =.1;
	protected static final double DEFAULT_INC_IN_DIST_IF_INVALID_PNT = 20; 
	protected static final boolean var_a = false;
	protected static final double DEFAULT_CM_OC_A = 0;   		
	protected static final double DEFAULT_CM_NOC_A = 0;	  		
	protected static final double DEFAULT_ZM_IZ_A = 0;	
	protected static final double DEFAULT_ZM_NIZ_A = 0; 	
	
	protected static final double DEFAULT_R = 3500; 
	protected static final double DEFAULT_R_CC = 0;
	protected static final double DEFAULT_RD = 2000; 
	protected static final double DEFAULT_RDb = 1000; 
	protected static final double DEFAULT_RD2 = 1500;
	protected static final double DEFAULT_RD3 = 1000;
	protected static final double DEFAULT_RD4 = 1000;
	protected static final double DEFAULT_RD5 = 50;

	protected static final double DEFAULT_ZONEAREA_ADAP = 0.0005;
	protected static final double DEFAULT_REP_NON_IC = 50; 
	protected static final double DEFAULT_ATT_IC_A = 0.005;
	
	protected boolean causeOnlyPosChange = false;
	protected boolean circularDistPntEdge = true; 
	protected boolean repOnContainingContours = true; 
	protected boolean repOnContainedContours = true; 
	protected boolean repPntsNonICs = true; 
	protected boolean repPntsICs = true;
	protected boolean repAllPnts = false;
	protected boolean clToPntRep = false;
	protected boolean zoneAreaAdap = true; 
	protected boolean zoneAreaAdap2 = false; 
	protected boolean attICs = false; 
	protected boolean repNonICs = false; 
	
	
	protected static final boolean var_r = false;
	protected static final double DEFAULT_CM_OC_R = 0;   		
	protected static final double DEFAULT_CM_NOC_R = 0;  		
	protected static final double DEFAULT_ZM_IZ_R = 0;	
	protected static final double DEFAULT_ZM_NIZ_R = 0;  		

	protected static final double DEFAULT_M = .6;      	

	protected static final double DEFAULT_INIT_MAX_ABS_FORCE = 100000; 
	protected static final double DEFAULT_MIN_MAX_ABS_FORCE = 10; 
	
	protected static final double DEFAULT_AT = 100;       
	protected static final double DEFAULT_RT = 20;      
	protected static final boolean at_on = false;        		
	protected static final boolean rt_on = false;        	
	
	protected int arrangeZoneAreasI = 25;
	protected boolean arrangeZoneAreas = false;
	protected boolean equilContourSize = false;

	
	
	// Object Data Fields/Variables

	//  No. of Iterations
	protected int i;
	
	//  Forces' Constants
	//    Attraction 
	protected double a;
	//    Attraction on centroid graph
	protected double a_cc = DEFAULT_A_CC;
	//    Attraction from Contour Internal Middle (for points on contour)
	protected double cm_oc_a;
	//    Attraction from Contour Internal Middle (for points not on contour)
	protected double cm_noc_a;
	//    Attraction from Zone Internal Middle (for points in zone)
	protected double zm_iz_a;
	//    Attraction from Zone Internal Middle (for points not in zone)
	protected double zm_niz_a;
	
	//    Repulsion 
	protected double r;
	//    Repulsion on centroid graph
	protected double r_cc = DEFAULT_R_CC;
	//    Repulsion between pnts on different contours
	protected double rd;
	//    Repulsion from Contour Internal Middle (for points on contour)
	protected double cm_oc_r;
	//    Repulsion from Contour Internal Middle (for points not on contour)
	protected double cm_noc_r;
	//    Repulsion from Zone Internal Middle (for points in zone)
	protected double zm_iz_r;
	//    Repulsion from Zone Internal Middle (for points not in zone)
	protected double zm_niz_r;
	
	//  Node Movement Constant
	protected double m;
	
	protected double maxAbsForce = DEFAULT_INIT_MAX_ABS_FORCE;
	protected double minMaxAbsForce = DEFAULT_MIN_MAX_ABS_FORCE;
	protected double at = DEFAULT_AT;
	protected double rt = DEFAULT_RT;
	
	
	private ArrayList<ConcreteContour> concreteContours;
	private SimpleConcreteDiagram simpleConcreteDiagram;
	private SimpleDiagramPanel simpleDiagramPanel;

	
	
	// Constructors
	public SimpleDiagramDrawerForceModel(){ 
		super(DEFAULT_ACCELERATORKEY, DEFAULT_MENUTEXT, DEFAULT_ACCELERATORKEY);
		setAll();
	}
	public SimpleDiagramDrawerForceModel(int inAcceleratorKey){ 
		super(inAcceleratorKey, DEFAULT_MENUTEXT, inAcceleratorKey);
		setAll();
	}
	public SimpleDiagramDrawerForceModel (int inAcceleratorKey, int inI, double inA, double inCM_OC_A, double inCM_NOC_A, double inZM_IZ_A, double inZM_NIZ_A, double inR, double inRD, double inCM_OC_R, double inCM_NOC_R, double inZM_IZ_R, double inZM_NIZ_R, double inM){
		super (inAcceleratorKey, DEFAULT_MENUTEXT, inAcceleratorKey);
		setAll (inI, inA, inCM_OC_A, inCM_NOC_A, inZM_IZ_A, inZM_NIZ_A, inR, inRD, inCM_OC_R, inCM_NOC_R, inZM_IZ_R, inZM_NIZ_R, inM);
	}
	
	//Properties (accessor & mutator methods)
	public int getI(){
		return i;
	}
	public void setI(int inI){
		i = inI;
	}
	public double getA(){
		return a;
	}
	public void setA(double inA){
		a = inA;
	}
	public double getCM_OC_A(){
		return cm_oc_a;
	}
	public void setCM_OC_A(double inCM_OC_A){
		cm_oc_a = inCM_OC_A;
	}
	public double getCM_NOC_A(){
		return cm_noc_a;
	}
	public void setCM_NOC_A(double inCM_NOC_A){
		cm_noc_a = inCM_NOC_A;
	}
	public double getZM_IZ_A(){
		return zm_iz_a;
	}
	public void setZM_IZ_A(double inZM_IZ_A){
		zm_iz_a = inZM_IZ_A;
	}	
	public double getZM_NIZ_A(){
		return zm_niz_a;
	}
	public void setZM_NIZ_A(double inZM_NIZ_A){
		zm_niz_a = inZM_NIZ_A;
	}
	public double getR(){
		return r;
	}
	public void setR(double inR){
		r = inR;
	}
	public double getRD(){
		return rd;
	}
	public void setRD(double inRD){
		rd = inRD;
	}
	public double getCM_OC_R(){
		return cm_oc_r;
	}
	public void setCM_OC_R(double inCM_OC_R){
		cm_oc_r = inCM_OC_R;
	}
	public double getCM_NOC_R(){
		return cm_noc_r;
	}
	public void setCM_NOC_R(double inCM_NOC_R){
		cm_noc_r = inCM_NOC_R;
	}
	public double getZM_IZ_R(){
		return zm_iz_r;
	}
	public void setZM_IZ_R(double inZM_IZ_R){
		zm_iz_r = inZM_IZ_R;
	}	
	public double getZM_NIZ_R(){
		return zm_niz_r;
	}
	public void setZM_NIZ_R(double inZM_NIZ_R){
		zm_niz_r = inZM_NIZ_R;
	}
	public double getM(){
		return m;
	}
	public void setM(double inM){
		m = inM;
	}
	public void setAll (){
		setI(DEFAULT_I);
		setA(DEFAULT_A);
		setCM_OC_A(DEFAULT_CM_OC_A);
		setCM_NOC_A(DEFAULT_CM_NOC_A);
		setZM_IZ_A(DEFAULT_ZM_IZ_A);
		setZM_NIZ_A(DEFAULT_ZM_NIZ_A);
		setR(DEFAULT_R);
		setRD(DEFAULT_RD);
		setCM_OC_R(DEFAULT_CM_OC_R);
		setCM_NOC_R(DEFAULT_CM_NOC_R);		
		setZM_IZ_R(DEFAULT_ZM_IZ_R);
		setZM_NIZ_R(DEFAULT_ZM_NIZ_R);
		setM(DEFAULT_M);
	}
	public void setAll (int inI, double inA, double inCM_OC_A, double inCM_NOC_A, double inZM_IZ_A, double inZM_NIZ_A, double inR, double inRD, double inCM_OC_R, double inCM_NOC_R, double inZM_IZ_R, double inZM_NIZ_R, double inM){
		setI(inI);
		setA(inA);
		setCM_OC_A(inCM_OC_A);
		setCM_NOC_A(inCM_NOC_A);
		setZM_IZ_A(inZM_IZ_A);
		setZM_NIZ_A(inZM_NIZ_A);
		setR(inR);
		setRD(inRD);
		setCM_OC_R(inCM_OC_R);
		setCM_NOC_R(inCM_NOC_R);
		setZM_IZ_R(inZM_IZ_R);
		setZM_NIZ_R(inZM_NIZ_R);
		setM(inM);
	}
	
	
	protected void setConcreteContours (ArrayList<ConcreteContour> inConcreteContours){
		concreteContours = inConcreteContours;
		for (ConcreteContour concreteContour : concreteContours){
			concreteContour.setContourLines();
		}
	}	
	public SimpleConcreteDiagram getSimpleConcreteDiagram(){
		return simpleConcreteDiagram;
	}
	public void setSimpleConcreteDiagram (SimpleConcreteDiagram inSimpleConcreteDiagram){
		simpleConcreteDiagram = inSimpleConcreteDiagram;
		setConcreteContours(simpleConcreteDiagram.getConcreteContours());
	}	
	public SimpleDiagramPanel getSimpleDiagramPanel(){
		return simpleDiagramPanel;
	}
	public void setSimpleDiagramPanel (SimpleDiagramPanel inSimpleDiagramPanel){
		simpleDiagramPanel = inSimpleDiagramPanel;
		super.simpleDiagramPanel = inSimpleDiagramPanel;
	}	
	
	
	//Max Absolute Force 
	protected void setMaxAbsForceForNextIteration (int n){
		if ((n > 0) && (maxAbsForce > minMaxAbsForce)){
			maxAbsForce = (maxAbsForce / Math.pow(n,1));} 
	}
	//Force functions
	protected double totalAttraction (double distance, Double contourRectAreaValue){
		double att;
		if (var_a){
			att =  a * (1/contourRectAreaValue) * distance;	
		} else {
			att =  a * distance;
		}
		if ((at_on)&&(distance > at)){
			return 0;
		} else {
			return att;
		}
	}
	protected double totalAttractionContourCentroid (double distance){
		return (a_cc * distance);	
	}
	protected double totalAttractionPntsOnDiffContours (double distance, Double choice){
		if (choice ==0){
			return (DEFAULT_AD * (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}else if (choice == 0.5){
			return (DEFAULT_ADb * (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}else if (choice ==1){
			return (DEFAULT_AD2 * (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}else if (choice ==2){
			return (DEFAULT_AD3 * (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}
		return (DEFAULT_AD * (distance));
	}
	protected double totalAttractionPntsOnDiffContours2 (double distance, Double choice){
		if (choice ==0){
			return (DEFAULT_AD / (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}else if (choice == 0.5){
			return (DEFAULT_ADb/ (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}else if (choice ==1){
			return (DEFAULT_AD2/ (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}else if (choice ==2){
			return (DEFAULT_AD3/ (distance + DEFAULT_INC_IN_DIST_IF_INVALID_PNT));
		}
		return (DEFAULT_AD * (distance));
	}
	protected double totalContourMiddleAttraction_PointOnContour (double distance){
		double att = (cm_oc_a * distance);
		if ((at_on)&&(distance > at)){
			return 0;
		} else {
			return att;
		}
	}	
	protected double totalContourMiddleAttraction_PointNotOnContour (double distance){
		double att = (cm_noc_a * distance);
		if ((at_on)&&(distance > at)){
			return 0;
		} else {
			return att;
		}
	}	
	protected double totalZoneMiddleAttraction_PointInZone  (double distance){
		double att = (zm_iz_a * distance);
		if ((at_on)&&(distance > at)){
			return 0;
		} else {
			return att;
		}
	}
	protected double totalZoneMiddleAttraction_PointNotInZone  (double distance){
		double att = (zm_niz_a * distance);
		if ((at_on)&&(distance > at)){
			return 0;
		} else {
			return att;
		}
	}	
	protected double totalRepulsion (double distance, Double contourRectAreaValue){
		double rep;
		if (var_r){
			rep =  (r*contourRectAreaValue) / ((double)(Math.pow(distance,2)));	
		} else {
			rep =  r / ((double)(Math.pow(distance,2)));
		}
		if ((rt_on)&&(distance < rt)){
			return 0;
		} else {
			return rep;
		}
	}
	protected double totalRepulsionContourCentroid (double distance){
		return (r_cc / ((double)(Math.pow(distance,2))));	
	}
	protected double totalRepulsionPntsOnDiffContours (double distance, Double choice){
		double rd = 0;
		if (choice == 0){
			rd = DEFAULT_RD;
		}else if (choice == 0.5){
			rd = DEFAULT_RDb;
		}else if (choice == 1){
			rd = DEFAULT_RD2;
		}else if (choice == 2){
			rd = DEFAULT_RD4;
		}else if (choice == 3){
			rd = DEFAULT_RD5;
		} else {
			rd = DEFAULT_RD3;
		}
		return (rd/(Math.pow(distance,2)));
	}
	protected double totalContourMiddleRepulsion_PointOnContour (double distance){
		double rep = (cm_oc_r / ((double)(Math.pow(distance,2))));
		if ((rt_on)&&(distance < rt)){
			return 0;
		} else{
			return rep;
		}
	}
	protected double totalContourMiddleRepulsion_PointNotOnContour (double distance){
		double rep = (cm_noc_r / ((double)(Math.pow(distance,2))));
		if ((rt_on)&&(distance < rt)){
			return 0;
		} else{
			return rep;
		}
	}
	protected double totalZoneMiddleRepulsion_PointInZone (double distance){
		double rep = (zm_iz_r / ((double)(Math.pow(distance,2))));
		if (distance < rt){
			return 0;
		} else{
			return rep;
		}
	}	
	protected double totalZoneMiddleRepulsion_PointNotInZone (double distance){
		double rep = (zm_niz_r / ((double)(Math.pow(distance,2))));
		if (distance < rt){
			return 0;
		} else{
			return rep;
		}
	}	
	
	
	
	
	//Methods 

	
	/**
	 * This needs to be implemented for all GraphDrawers.
	 * Its executed when the accelerator key is pressed.
	 */

	//	overrides parent GraphDrawer and is overridden by its inheriting children 
	
	public void layout() {
		
		setSimpleConcreteDiagram(simpleDiagramPanel.getSimpleConcreteDiagram());

		String initialAbstractDesc = ConcreteContour.generateAbstractDiagramFromList(concreteContours);
	
		// Equalise the number of points each contour has
		setConcreteContours(equalizePointsOnContours(concreteContours));

		// Find the zones in which the points of the original contours reside
		ArrayList<Zone> origZones = new ArrayList<Zone>();
		HashMap<String, ArrayList<Zone>> origZonesForPnts = new HashMap<String, ArrayList<Zone>>();
		if (repNonICs || circularDistPntEdge || zoneAreaAdap){
			origZones = getZones(concreteContours, false);
			if (repNonICs || circularDistPntEdge){
				origZonesForPnts = getZonesForContourPoints(concreteContours, origZones, false);
			}			
		}
		
		// Find all containments
		HashMap<ConcreteContour, ArrayList<ConcreteContour>> contoursContainingContours = new HashMap<ConcreteContour, ArrayList<ConcreteContour>>();
		if (repOnContainingContours || repPntsICs || attICs || circularDistPntEdge || zoneAreaAdap){
			contoursContainingContours = getContoursContainingContours(concreteContours);
		}
		
		HashMap<ConcreteContour, ArrayList<ConcreteContour>> contoursContainedInContours = new HashMap<ConcreteContour, ArrayList<ConcreteContour>>();
		if (zoneAreaAdap || repOnContainedContours){
			for (ConcreteContour cc : concreteContours){
				ArrayList<ConcreteContour> ccContains = contoursContainingContours.get(cc);
				if (ccContains != null){
					for (ConcreteContour containedContour : ccContains){
						if (!contoursContainedInContours.containsKey(containedContour)){
							ArrayList<ConcreteContour> containedInList = new ArrayList<ConcreteContour>();
							containedInList.add(cc);
							contoursContainedInContours.put(containedContour, containedInList);
						} else if (!contoursContainedInContours.get(containedContour).contains(cc)){
							contoursContainedInContours.get(containedContour).add(cc);							
						}	
					}
				}
			}
		}
		
		
		// get contours not intersecting with current contours (based on the original placement)
		ArrayList<ArrayList<ConcreteContour>> contoursNotIntersectingWithContour_ForAllContours = new ArrayList<ArrayList<ConcreteContour>>();
		if (repNonICs || repPntsNonICs){
			for (ConcreteContour cc : concreteContours){
				ArrayList<ConcreteContour> contoursNotIntersectingWithContour_ForContour = new ArrayList<ConcreteContour>();
				ArrayList<ConcreteContour> ics = getIntersectingContours(cc, concreteContours, origZones);
				for (ConcreteContour cc2 : concreteContours){
					if (cc == cc2) continue;
					if (!ics.contains(cc2)){
						contoursNotIntersectingWithContour_ForContour.add(cc2);
					}
				}
				contoursNotIntersectingWithContour_ForAllContours.add(contoursNotIntersectingWithContour_ForContour);
			}
		}
		
		
		
		// get all intersecting contours for all the contours (based on the original placement)
		ArrayList<ArrayList<ConcreteContour>> intersectingContoursForAllContours = new ArrayList<ArrayList<ConcreteContour>>();
		if (attICs || clToPntRep || repAllPnts || repPntsICs){
			intersectingContoursForAllContours = new ArrayList<ArrayList<ConcreteContour>>();
			for (ConcreteContour cc : concreteContours){
				intersectingContoursForAllContours.add(getIntersectingContours(cc, concreteContours)); 
			}
		}		

		
		// Calculate some things to get proportional zone areas
		ArrayList<String> contourAbsDesc = new ArrayList<String>();
		double[] nSetZoneAreaProp = new double[concreteContours.size()];
		ArrayList<String>[] origZonesInContour = new ArrayList [concreteContours.size()];
		int maxNoOfSetsInZones = 0;
		if (zoneAreaAdap){
			
			// Store description of contours 
			// -> using the indexOf method of arraylist, would be easier to retrieve the index of the contour
			for (ConcreteContour cc : concreteContours){
				contourAbsDesc.add(cc.getAbstractContour());
			}	
			
			
			// Calculate number of n-set zones & find the maximum number of sets used for zones 
			double[] nSetZonesCount = new double[concreteContours.size()];
			for (int j=0; j<nSetZonesCount.length; j++){  //initialize
				nSetZonesCount[j] = 0;
			}
			maxNoOfSetsInZones = 0;
			for (Zone z : origZones){
				int zNoOfSets = z.getAbstractDescription().length();
				if (zNoOfSets > maxNoOfSetsInZones){
					maxNoOfSetsInZones = zNoOfSets;
				}
				nSetZonesCount [zNoOfSets-1] ++;
			}
			
			
			// Calculates weights for each n-set zone
			double[] nSetZoneWeight = new double[concreteContours.size()];
			int weightedZoneCount = 0;
			for (int j=0; j < nSetZoneWeight.length; j++){
				nSetZoneWeight[j] = Math.pow(2,maxNoOfSetsInZones)/Math.pow(2,(j+1));
				weightedZoneCount += (nSetZoneWeight[j] * nSetZonesCount[j]);				
			}
			
			
			// Get zone area proportions 
			for (int j=0; j < nSetZoneAreaProp.length; j++){
				nSetZoneAreaProp[j] = nSetZoneWeight[j] / weightedZoneCount;				
			}
	
			
			// Get description of zones in each contour
			for (int j =0; j<origZonesInContour.length; j++){ // initialize
				origZonesInContour[j] = new ArrayList<String>();
			}
			for (Zone z: origZones){
				String zAbsDesc = z.getAbstractDescription();
				for (int j =0; j < zAbsDesc.length(); j++){
					int cci = contourAbsDesc.indexOf(Character.toString(zAbsDesc.charAt(j)));
					origZonesInContour[cci].add(zAbsDesc);
				}
			}
		}
		
	
		
		// Iterate & move pnts i times
		simpleDiagramPanel.getStopWatch().start();
		i = DEFAULT_I_for3;
		a = DEFAULT_A_for3;
		if (simpleConcreteDiagram.getConcreteContours().size() == 4){
			i = DEFAULT_I_for4;
			a = DEFAULT_A_for4;
		} else if(simpleConcreteDiagram.getConcreteContours().size() == 5){
			i = DEFAULT_I_for5;
			a = DEFAULT_A_for5;
		}
		
		
		
		for (int c = 0; c < i; c++){  
			
			// Compute Max Absolute Force for this Iteration
			setMaxAbsForceForNextIteration(c);
			
			// Forces for Points on the Contours
			ArrayList<ArrayList<Point2D.Double>> xyForceOnContoursPoints = new ArrayList<ArrayList<Point2D.Double>>();
			
			//ArrayList<ArrayList<Point2D.Double[]>> xyForceOnContoursLines = new ArrayList<ArrayList<Point2D.Double[]>>();
			ArrayList<ArrayList<Point2D.Double>> xyForceOnContoursLines = new ArrayList<ArrayList<Point2D.Double>>();
			
			ArrayList<Zone> currentZones = new ArrayList<Zone>();
			HashMap<String,ArrayList<Zone>> zonesForContourPnts = new HashMap<String, ArrayList<Zone>>();
			if (circularDistPntEdge  || zoneAreaAdap){
				currentZones = getZones(concreteContours, false);
				if (circularDistPntEdge){
					zonesForContourPnts = getZonesForContourPoints(concreteContours, currentZones);
				}
			}
			
			
			double[] avgZoneAreasForNSets = null; 
			//set the size of the array to the max N sets zone you can have
			if (zoneAreaAdap2){
				avgZoneAreasForNSets = getAvgZoneAreasForNSets(concreteContours, currentZones);
			}
			
		
			// Calculate some things to get proportional zone areas
			double zoneAreaAdapProgChange = 0; 
			double maxCInc = Math.pow(concreteContours.size(), 2) * 10; 
			double constAtMaxCInc = 0.03;
			double pow = 0.5;
			double constInc = constAtMaxCInc / (Math.pow(maxCInc, pow)); 
			if ((c+1) < maxCInc){
				zoneAreaAdapProgChange = constInc * Math.pow((c+1), pow); 
			}else {
				zoneAreaAdapProgChange = constAtMaxCInc;
			}
			
			ArrayList<Zone>[] currZonesInContours = new ArrayList [concreteContours.size()];
			ArrayList<String> currZonesToConsiderNames = new ArrayList<String>();  // to retrieve index of that zone for quick retrieval from other lists
			ArrayList<Zone> currZonesToConsider = new ArrayList<Zone>(); 
			ArrayList<Zone>[] currInvalidZonesInContours = new ArrayList [concreteContours.size()];
			ArrayList<String>[] currMissingZonesInContours = new ArrayList [concreteContours.size()];
			double[] incZonesAreaBy = null; 
			double[] incZonesAreaByWithImpFactor = null; 
			double[] currNSetZonesPropArea = new double[concreteContours.size()];
			double[] idealNSetZonesPropArea = new double[concreteContours.size()];
			double[] contoursEnlargeFactor = new double [concreteContours.size()]; 
			double[] incContoursAreaBy = new double[concreteContours.size()];
			if (zoneAreaAdap && (concreteContours.size()>0)){
				
				// Get current contour areas 
				double[] currContourArea = new double[concreteContours.size()];
				double totalCurrContoursArea = 0;
				int cci=0;
				for (ConcreteContour cc : concreteContours){
					currContourArea[cci] = computeContourArea(cc);
					totalCurrContoursArea += currContourArea[cci];
					cci++;
				}
				
				// Get proportional zone areas for n-set zones using current total contour areas
				double incZoneAreaBy = 0;
				for (int j=0; j<currNSetZonesPropArea.length; j++){ 
					currNSetZonesPropArea[j] =  nSetZoneAreaProp[j] * totalCurrContoursArea;
					double minZoneArea = 4000 * (maxNoOfSetsInZones - j);// in full 5000 * (maxNoOfSetsInZones-(j+1)+1);
					double aboveMinZoneArea = currNSetZonesPropArea[j] - minZoneArea;
					if ((aboveMinZoneArea < 0) && (Math.abs(aboveMinZoneArea) > incZoneAreaBy)){ 
						incZoneAreaBy = Math.abs(aboveMinZoneArea);
					}
				}
				
				
				// Get ideal zone areas for n-set zones 
				for (int j=0; j<idealNSetZonesPropArea.length; j++){ 
					idealNSetZonesPropArea[j] = currNSetZonesPropArea[j] + incZoneAreaBy;
				}
				

				// Get actual zones for contours (get only those zones mentionned in original description)
				for (int j =0; j<currZonesInContours.length; j++){ // initialize
					currZonesInContours[j] = new ArrayList<Zone>();
				}
				for (int j =0; j<currInvalidZonesInContours.length; j++){ // initialize
					currInvalidZonesInContours[j] = new ArrayList<Zone>();
				}
				for (Zone z: currentZones){
					String zAbsDesc = z.getAbstractDescription();
					for (int j =0; j < zAbsDesc.length(); j++){
						cci = contourAbsDesc.indexOf(Character.toString(zAbsDesc.charAt(j)));
						if (origZonesInContour[cci].contains(zAbsDesc)){
							if (!currZonesInContours[cci].contains(z)){
								currZonesInContours[cci].add(z);
							}
							// check if already available just in case have duplicates
							if (!currZonesToConsider.contains(z) && !currZonesToConsiderNames.contains(zAbsDesc)){
								currZonesToConsider.add(z);
								currZonesToConsiderNames.add(zAbsDesc);
							}
						} else { // if zone not in original 
							if (!currInvalidZonesInContours[cci].contains(z)){
								currInvalidZonesInContours[cci].add(z);
							}
						}
					}
				}
				for (int j =0; j<currMissingZonesInContours.length; j++){ // initialize
					currMissingZonesInContours[j] = new ArrayList<String>();
				}
				for (int j = 0; j<concreteContours.size(); j++){
					ArrayList<String> origZonesAbsDescInContour = origZonesInContour[j];
					
					for (String origZoneAbsDesc : origZonesAbsDescInContour){
						ArrayList<Zone> currZonesInContour = currZonesInContours[j];
						boolean zoneCurrentlyAvailable = false;
						for (Zone cz : currZonesInContour){
							if (origZoneAbsDesc.equals(cz.getAbstractDescription())){
								zoneCurrentlyAvailable = true;
								break;
							}
						}
						if (!zoneCurrentlyAvailable){
							currMissingZonesInContours[j].add(origZoneAbsDesc);
						}
					}
				}
				
				
				
				// Get ideal contour areas and the required contour increase in area 
				double[] idealContourArea = new double[concreteContours.size()];
				for (int j=0; j < concreteContours.size(); j++){
					idealContourArea[j] = 0;
					for (Zone z : currZonesInContours[j]){
						idealContourArea[j] += idealNSetZonesPropArea [z.getAbstractDescription().length()-1];
					}
					for (String zDesc : currMissingZonesInContours[j]){
						idealContourArea[j] += idealNSetZonesPropArea [zDesc.length()-1];
					}
					incContoursAreaBy[j] = idealContourArea[j] - currContourArea[j];
				}
				
				
				// Calculate required increase in zone areas 
				incZonesAreaBy = new double[currZonesToConsider.size()];
				incZonesAreaByWithImpFactor = new double[currZonesToConsider.size()];
				int zi=0;
				for (Zone z : currZonesToConsider){
					int noOfSetsInZone = z.getAbstractDescription().length();
					double zCurrArea = z.computeTotalArea();
					incZonesAreaBy[zi] = idealNSetZonesPropArea[noOfSetsInZone-1] - zCurrArea;
					zi++;
				}
				
				
				// Enlarge contours 
				contoursEnlargeFactor = null;
			}
			
			
			//
			ArrayList<Point2D.Double> xyForceOnContours = new ArrayList<Point2D.Double>();
			Point2D.Double[] xyOtherForceOnContours = new Point2D.Double[concreteContours.size()];
			for (int j=0; j<xyOtherForceOnContours.length; j++){ // initialize
				xyOtherForceOnContours[j] = new Point2D.Double(0,0);
			}
			
			
			// attract entirely contained contours to the centre of the contour/zone they are contained in 
			if (zoneAreaAdap){
				
				for (ConcreteContour cc : concreteContours){
					ArrayList<ConcreteContour> contoursContainedInCC = contoursContainingContours.get(cc);
					if ((contoursContainedInCC == null) || (contoursContainedInCC.size()<1)){continue;}
					
					
					HashMap<String, ArrayList<ConcreteContour>> contoursInSameAreas = new HashMap<String, ArrayList<ConcreteContour>>();
					for (ConcreteContour contourContainedInCC : contoursContainedInCC){
						ArrayList<ConcreteContour> contourContainedInContours = contoursContainedInContours.get(contourContainedInCC);
						boolean[] csContainingContour = new boolean[concreteContours.size()];
						for (int j =0; j< csContainingContour.length; j++){ //initialize
							csContainingContour[j] = false;
						}
						for (ConcreteContour cic : contourContainedInContours){
							csContainingContour[contourAbsDesc.indexOf(cic.getAbstractContour())] = true;
						}
						String csContainingContourStr = "";
						for (int j =0; j< csContainingContour.length; j++){
							if (csContainingContour[j]){
								csContainingContourStr += contourAbsDesc.get(j);
							}
						}
						if (contoursInSameAreas.containsKey(csContainingContourStr) && 
							!contoursInSameAreas.get(csContainingContourStr).contains(contourContainedInCC)){
							contoursInSameAreas.get(csContainingContourStr).add(contourContainedInCC);
						} else {
							ArrayList<ConcreteContour> contourList = new ArrayList<ConcreteContour>();
							contourList.add(contourContainedInCC);
							contoursInSameAreas.put(csContainingContourStr, contourList);
						}
					}
					
					Iterator itr = contoursInSameAreas.entrySet().iterator();				
					while (itr.hasNext()){
						Entry<String, ArrayList<ConcreteContour>> contoursInSameArea = (Entry<String, ArrayList<ConcreteContour>>)itr.next();
						
						String containingContoursStr = contoursInSameArea.getKey();
						ArrayList<ConcreteContour> contourListInSameArea = contoursInSameArea.getValue();
						
						Area a1 = new Area();
						for (ConcreteContour cisa : contourListInSameArea){
							Area a2 = new Area(cisa.getPolygon());
							a1.add(a2);
						}
						
						ArrayList<Polygon> containedPolys = (ConcreteContour.polygonsFromArea(a1));
						Point2D.Double polyUnionCentroid = new Point2D.Double(0,0);
						if (containedPolys.size() == 1){
							polyUnionCentroid = convertToPoint2D_Double(Utilities.findMiddlePointInsidePolygon(containedPolys.get(0)));
						}if (containedPolys.size() > 1){
							for (Polygon containedPoly : containedPolys){
								Point2D.Double polyCentroid = convertToPoint2D_Double(Utilities.findMiddlePointInsidePolygon(containedPoly));
								polyUnionCentroid.x += polyCentroid.x;
								polyUnionCentroid.y += polyCentroid.y;
							}
							polyUnionCentroid.x = polyUnionCentroid.x/containedPolys.size();
							polyUnionCentroid.y = polyUnionCentroid.y/containedPolys.size();
						}
						
						
						//check if contour it is contained in is in turn contained in other contour -> if so remove common contours
						String revisedContainingContoursStr = containingContoursStr;
						if (containingContoursStr.length() > 1) { 
							for (int j =0; j< containingContoursStr.length(); j++){
								ArrayList<ConcreteContour> csOtherContourContainedIn = contoursContainedInContours.get(concreteContours.get(contourAbsDesc.indexOf(Character.toString(containingContoursStr.charAt(j)))));
								if (csOtherContourContainedIn == null){continue;}
								for (ConcreteContour ocCI : csOtherContourContainedIn){
									int ocInStrIndex = revisedContainingContoursStr.indexOf(ocCI.getAbstractContour());
									if (ocInStrIndex <0){continue;}
									revisedContainingContoursStr = revisedContainingContoursStr.substring(0,ocInStrIndex)+revisedContainingContoursStr.substring(ocInStrIndex+1);
								}
							}
						}
				
						double forceConst = 0;
						if (revisedContainingContoursStr.length() < 1) { continue;} 
						Point2D.Double attractingPnt = new Point2D.Double (0,0);
						if (revisedContainingContoursStr.length() == 1){
							attractingPnt = computeContourCentroid(concreteContours.get(contourAbsDesc.indexOf(revisedContainingContoursStr)));
							
							double cMaxInc = (Math.pow(concreteContours.size(),2)*10); 
							double maxConst = 0.05;
							if ((c+1) < cMaxInc){
								forceConst = (maxConst / cMaxInc) * (c+1);
							} else{
								forceConst = maxConst;
							}
							forceConst = 0.03;
						} else {
							
							int containingZoneIndex = currZonesToConsiderNames.indexOf(revisedContainingContoursStr);
							if (containingZoneIndex < 0){continue;}
							Zone containingZone = currZonesToConsider.get(containingZoneIndex);
							if (containingZone == null){continue;}
							attractingPnt = containingZone.computeCentroid(); // might have disconnect zones
							
							double cMaxInc = (Math.pow(concreteContours.size(),2)*10); 
							double maxConst = 0.05;
							if ((c+1) < cMaxInc){
								forceConst = (maxConst / cMaxInc) * (c+1);
							} else{
								forceConst = maxConst;
							}
							forceConst = 0.01;
						}
							
						if (containedPolys.size() >= 1){
							Point2D.Double dir = getForceDirection (ForceType.a, polyUnionCentroid, attractingPnt);
							Point2D.Double xyDistance = getxyAbsDistances (polyUnionCentroid, attractingPnt);
							double distance = getAbsHypotenuse(xyDistance);
				
							Point2D.Double xyForce = new Point2D.Double (0,0);
							xyForce.x = dir.x * (xyDistance.x/(xyDistance.x+xyDistance.y)) * distance * forceConst; 
							xyForce.y = dir.y * (xyDistance.y/(xyDistance.x+xyDistance.y)) * distance * forceConst; 
				
							for (ConcreteContour inc : contourListInSameArea){
								int incIndex = contourAbsDesc.indexOf(inc.getAbstractContour());
								xyOtherForceOnContours[incIndex].x += xyForce.x;
								xyOtherForceOnContours[incIndex].y += xyForce.y;
							}
						}
					}
				}
			}
			
			
			int cci =0;
			for (ConcreteContour cc : concreteContours){
				
				double aInc = 0;
				double rInc = 0;
				
				double rInc2=0;
				double aInc2=0;
				
				ArrayList<Point2D.Double> xyForceContourPoints = new ArrayList<Point2D.Double>();
		
				Polygon cPoly = cc.getPolygon();
				ArrayList<ContourLine> contourLines = cc.getContourLines();
	
				Point2D.Double xyForceOnContour = new Point2D.Double(0,0);
				
				if (zoneAreaAdap){
					
					ArrayList<ConcreteContour> contoursContainedInCC = contoursContainingContours.get(cc);
					
					double cMaxInc = (Math.pow(concreteContours.size(),2)*10); 
					double maxConst = 0.13;
					if ((c+1) < cMaxInc){
						zoneAreaAdapProgChange = (maxConst / cMaxInc) * (c+1);
					} else{
						zoneAreaAdapProgChange = maxConst;
					}
					
					for (Zone z : currZonesInContours[cci]){
						
						String zAbsDesc = z.getAbstractDescription();
						
						// move contour only if the zone area will directly be affected by the movement of the contour
						boolean zAffectedWithContourMove = true;

						if (contoursContainedInCC != null){
							for (int j =0; j<zAbsDesc.length(); j++){
								if (contoursContainedInCC.contains(Character.toString(zAbsDesc.charAt(j)))){
									zAffectedWithContourMove = false;
									break;
								}
							}
						}
						
						// move contour wrt to zone
						if (zAffectedWithContourMove){
							double totalForce = 0.0006 * zoneAreaAdapProgChange * incZonesAreaBy[currZonesToConsiderNames.indexOf(zAbsDesc)]; 
							
							if (z.computeCentroid() == null){continue;}
							
							Point2D.Double dir = getForceDirection (ForceType.a, computeContourCentroid(cc), z.computeCentroid());
							Point2D.Double xyDistance = getxyAbsDistances (computeContourCentroid(cc), z.computeCentroid());
							
							Point2D.Double xyForce = new Point2D.Double (0,0);
							xyForce.x = dir.x * (xyDistance.x/(xyDistance.x+xyDistance.y)) * totalForce;
							xyForce.y = dir.y * (xyDistance.y/(xyDistance.x+xyDistance.y)) * totalForce;
							
							xyForceOnContour.x += xyForce.x;
							xyForceOnContour.y += xyForce.y;
						}				
					}
					
					for (Zone z : currInvalidZonesInContours[cci]){
						double totalForce = z.computeTotalArea() * zoneAreaAdapProgChange * 0.002; 
						
						if ((computeContourCentroid(cc)==null) ||  (z.computeCentroid()==null)){continue;}
						Point2D.Double dir = getForceDirection (ForceType.r, computeContourCentroid(cc), z.computeCentroid());
						Point2D.Double xyDistance = getxyAbsDistances (computeContourCentroid(cc), z.computeCentroid());
						
						Point2D.Double xyForce = new Point2D.Double (0,0);
						xyForce.x = dir.x * (xyDistance.x/(xyDistance.x+xyDistance.y)) * totalForce; 
						xyForce.y = dir.y * (xyDistance.y/(xyDistance.x+xyDistance.y)) * totalForce; 
						
						xyForceOnContour.x += xyForce.x;
						xyForceOnContour.y += xyForce.y;
					}
					
					for (String missingZoneAbsDesc : currMissingZonesInContours[cci]){
						missingZoneAbsDesc = missingZoneAbsDesc.replace(cc.getAbstractContour(), "");
						
						Point2D.Double xyForce = new Point2D.Double (0,0);
						
						for (int j=0; j<missingZoneAbsDesc.length(); j++){
							String contourDesc = Character.toString(missingZoneAbsDesc.charAt(j));
							int contourIndex = contourAbsDesc.indexOf(contourDesc);
							ConcreteContour oc = concreteContours.get(contourIndex);
							
							Point2D.Double dir = getForceDirection (ForceType.a, computeContourCentroid(cc), computeContourCentroid(oc));
							Point2D.Double xyDistance = getxyAbsDistances (computeContourCentroid(cc), computeContourCentroid(oc));
							double distance = getAbsHypotenuse(xyDistance);
							double totalforce = 0.05 * distance;
							
							xyForce.x += dir.x * (xyDistance.x/(xyDistance.x+xyDistance.y)) * totalforce; 
							xyForce.y += dir.y * (xyDistance.y/(xyDistance.x+xyDistance.y)) * totalforce; 
						}
						
						xyForce.x = xyForce.x / missingZoneAbsDesc.length();
						xyForce.y = xyForce.y / missingZoneAbsDesc.length();
						
						xyForceOnContour.x += xyForce.x;
						xyForceOnContour.y += xyForce.y;
					}
					
					rInc = zoneAreaAdapProgChange * incContoursAreaBy[cci];
				}		

			
			
				//Calculate all the forces on each point of this contour
				for (int p=0; p < cPoly.npoints; p++)
				{				
					Point2D.Double pnt = new Point2D.Double (cPoly.xpoints[p], cPoly.ypoints[p]);
					Point2D.Double pnt2;
					
					Point2D.Double xyForceOnPnt = new Point2D.Double(0,0);
			
						
					
					if (circularDistPntEdge){
						int oci =-1;
						for (ConcreteContour oc : concreteContours){
							oci++;
							if (cc == oc){continue;}
							
							oc.setContourLines();
							ArrayList<ContourLine> ocContourLines = oc.getContourLines();
							
							Point2D.Double selCL_xyDist = new Point2D.Double(0,0);
							double selCL_dist = -1;
							ContourLine selCL = null;
							Point2D.Double selCL_perpPntOnCL = new Point2D.Double();
							int ocli =-1;
							for (ContourLine cl :  ocContourLines){
								ocli++;
								MyLine clLine = new MyLine (cl.getLine());
								
								
								selCL_perpPntOnCL = clLine.getIntPntWithPerpLine(pnt);
								if (!clLine.isPointOnLineSeg(selCL_perpPntOnCL, new Point2D.Double (0,0))){continue;} 
							
								selCL_xyDist = getxyAbsDistances(pnt, selCL_perpPntOnCL);
								selCL_dist = getAbsHypotenuse(selCL_xyDist);
								selCL=cl;
								
								Point2D.Double new_xyForce = new Point2D.Double(0,0);
								Point2D.Double dir = new Point2D.Double(0,0);
								//find line-point relationship
								double distIfIncorrect = 0; 
								
								
							   	distIfIncorrect = selCL_dist + DEFAULT_INC_IN_DIST_IF_INVALID_PNT;
								
				
								if ((contoursContainingContours.get(oc)!=null)&& (contoursContainingContours.get(oc).contains(cc))){
									
									boolean innerPntIsValid = isPointInPolygon(oc.getPolygon(), pnt);
									boolean outerPntIsValid = !isPointInPolygon(cc.getPolygon(), selCL_perpPntOnCL);
									
									double totalForce =0.0;
									if ((innerPntIsValid && outerPntIsValid) ) {
										dir = getForceDirection (ForceType.r, pnt, selCL_perpPntOnCL);										
										totalForce = DEFAULT_RDb/Math.pow(selCL_dist, 2);		
									}
									else { 
										if ( (!innerPntIsValid && !outerPntIsValid) || (!innerPntIsValid && outerPntIsValid)){ //attraction only if the point on outer contour is also invalid
											dir = getForceDirection (ForceType.a, pnt, selCL_perpPntOnCL); 
											totalForce = DEFAULT_ADb * distIfIncorrect; 
										}
									}
									
									new_xyForce.x =  dir.x * (selCL_xyDist.x/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
									new_xyForce.y =  dir.y * (selCL_xyDist.y/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
									
								}
								else if ((contoursContainingContours.get(cc)!=null)&& (contoursContainingContours.get(cc).contains(oc))){
									
									double totalForce =0.0;
									
									boolean innerPntIsValid = isPointInPolygon(cc.getPolygon(), selCL_perpPntOnCL);
									boolean outerPntIsValid = !isPointInPolygon(oc.getPolygon(), pnt);
									
									
									// if the point on the outer contour is valid and the point on the inner contour is valid  OR
									//    the point on the outer contour is not valid but the point on the inner contour is valid
									if ( (outerPntIsValid && innerPntIsValid) ){ 
										dir = getForceDirection (ForceType.r, pnt, selCL_perpPntOnCL);
										totalForce = DEFAULT_RD/Math.pow(selCL_dist, 2);
									}
									else{ // if point on the outer contour is not valid and the point on the inner contour is also not valid
										if ( !outerPntIsValid && !innerPntIsValid){
											dir = getForceDirection (ForceType.a, pnt, selCL_perpPntOnCL); 
											totalForce = DEFAULT_AD * distIfIncorrect;
										}
									}
									new_xyForce.x =  dir.x * (selCL_xyDist.x/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
									new_xyForce.y =  dir.y * (selCL_xyDist.y/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
								}
								else if (!doContoursIntersect(cc, oc, origZones)){ 
									
									Zone commonZone = null;
									Zone smallestSplitZoneInCC = null;
									Zone smallestSplitZoneInOC = null;
									if (doContoursIntersect(cc, oc, currentZones)){
										ArrayList<Zone> currZonesInCC = currZonesInContours[cci];
										ArrayList<Zone> currZonesInOC = currZonesInContours[concreteContours.indexOf(oc)];
										
										String ccDesc = cc.getAbstractContour();
										String ocDesc = oc.getAbstractContour();
										
										//get common zone
										for (Zone zCC : currZonesInCC){
											if (currZonesInOC.contains(zCC)){
												commonZone = zCC;
												break;
											}
										}
										if (commonZone != null){
											String commonZoneDesc = commonZone.getAbstractDescription();
											String splitZoneInCC_desc = commonZoneDesc.replace(ocDesc, "");
											String splitZoneInOC_desc = commonZoneDesc.replace(ccDesc, "");
											
											double smallestSplitZoneInCCarea = 0;
											for (Zone zCC : currZonesInCC){
												if (zCC.getAbstractDescription().equals(splitZoneInCC_desc)){
													double zoneArea = zCC.computeTotalArea();
													if ((smallestSplitZoneInCC == null) || (smallestSplitZoneInCCarea > zoneArea)){
														smallestSplitZoneInCC = zCC;
														smallestSplitZoneInCCarea = zoneArea;
													}
												}
											}
											double smallestSplitZoneInOCarea = 0;
											for (Zone zOC : currZonesInOC){
												if (zOC.getAbstractDescription().equals(splitZoneInOC_desc)){
													double zoneArea = zOC.computeTotalArea();
													if ((smallestSplitZoneInOC == null) || (smallestSplitZoneInOCarea > zoneArea)){
														smallestSplitZoneInOC = zOC;
														smallestSplitZoneInOCarea = zoneArea;
													}
												}
											}
										}
									}
									
									
									double totalForce =0;
									if (!doContoursIntersect(cc, oc, currentZones)){
										dir = getForceDirection (ForceType.r, pnt, selCL_perpPntOnCL);
										totalForce = DEFAULT_RD2/Math.pow(selCL_dist, 2);
									}else{										
										boolean pntIsValid = false;
										boolean pnt2IsValid = false;
										
										if ((commonZone != null) && (smallestSplitZoneInCC != null) && (smallestSplitZoneInOC != null)){
											pntIsValid = (!smallestSplitZoneInCC.containsPoint(pnt)) || (!isPointInPolygon(oc.getPolygon(), pnt));
											pnt2IsValid = (!smallestSplitZoneInOC.containsPoint(selCL_perpPntOnCL)) || (!isPointInPolygon(cc.getPolygon(), selCL_perpPntOnCL));
										} else {
											pntIsValid = !isPointInPolygon(oc.getPolygon(), pnt);
											pnt2IsValid = !isPointInPolygon(cc.getPolygon(), selCL_perpPntOnCL);
										}
										
										if ((!pntIsValid && pnt2IsValid) ){ 
											dir = getForceDirection (ForceType.r, pnt, selCL_perpPntOnCL);
											totalForce = DEFAULT_RD2/Math.pow(selCL_dist, 2);
										} else if (!pntIsValid && !pnt2IsValid && ( (smallestSplitZoneInOC == null) || (smallestSplitZoneInOC.containsPoint(selCL_perpPntOnCL)) )) {
											dir = getForceDirection (ForceType.a, pnt, selCL_perpPntOnCL); 
											totalForce = DEFAULT_AD2 * distIfIncorrect; 
										}
									}
									
									new_xyForce.x =  dir.x * (selCL_xyDist.x/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
									new_xyForce.y =  dir.y * (selCL_xyDist.y/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
								}
								else if (doContoursIntersect(cc, oc, origZones) &&
										 ( ((contoursContainingContours.get(cc)!=null)&&(contoursContainingContours.get(cc).contains(oc))) ||
										   ((contoursContainingContours.get(oc)!=null)&&(contoursContainingContours.get(oc).contains(cc))) )
										){
									
									double totalForce =0;
									if (!doContoursIntersect(cc, oc, currentZones)){ 
										dir = getForceDirection (ForceType.a, pnt, selCL_perpPntOnCL);
										totalForce = DEFAULT_AD3 * distIfIncorrect;	
										
									}else{
										
										boolean pntInPoly2 = isPointInPolygon(oc.getPolygon(), pnt);
										boolean pnt2InPoly = isPointInPolygon(cc.getPolygon(), selCL_perpPntOnCL);
										
										if (pntInPoly2 && pnt2InPoly){									
											totalForce = DEFAULT_RD3/Math.pow(selCL_dist,2);
										} else if (!pntInPoly2 && pnt2InPoly){
											totalForce = DEFAULT_RD4 / Math.pow(selCL_dist,2);
										} else if (pntInPoly2 && !pnt2InPoly){
											totalForce = DEFAULT_RD5 / Math.pow(selCL_dist,2);
										}
										dir = getForceDirection (ForceType.r, pnt, selCL_perpPntOnCL);
										
										new_xyForce.x =  dir.x * (selCL_xyDist.x/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
										new_xyForce.y =  dir.y * (selCL_xyDist.y/(selCL_xyDist.x+selCL_xyDist.y)) * totalForce;
									}
								}
									
								// Check for Infinite or NaN
								
								if (Double.isInfinite(new_xyForce.x)){
									new_xyForce.x = dir.x * maxAbsForce;
								} else if (Double.isNaN(new_xyForce.x)){
									new_xyForce.x = 0;
								}
								
								if (Double.isInfinite(new_xyForce.y)){
									new_xyForce.y = dir.y * maxAbsForce;
								} else if (Double.isNaN(new_xyForce.y)){
									new_xyForce.y = 0;
								}
								xyForceOnPnt.x += new_xyForce.x;
								xyForceOnPnt.y += new_xyForce.y;
							}
						}
					}
					
					
					//calculate repulsive force
					//    in relation with every other point ON THIS contour
					r+=(rInc + rInc2);
					for (int p2=0; p2 < cPoly.npoints; p2++)
					{
						//if refers to same node then skip
						if (p2 == p){
							continue;}
						
						pnt2 = new Point2D.Double (cPoly.xpoints[p2], cPoly.ypoints[p2]);
						
						xyForceOnPnt = addxyRepulsion(xyForceOnPnt, pnt, pnt2, 0.0); 
					}
					r-=(rInc + rInc2);
					
					
					if (repPntsICs){
						ArrayList<ConcreteContour> intersectingContours = intersectingContoursForAllContours.get(cci);
						ArrayList<ConcreteContour> contoursInCC = contoursContainingContours.get(cc);
						
						if (intersectingContours != null){
							for (ConcreteContour ic : intersectingContours){
								
								ArrayList<ConcreteContour> contoursInIC = contoursContainingContours.get(ic);
								
								if ( ((contoursInCC != null) && (contoursInCC.contains(ic))) ||
									 ((contoursInIC != null) && (contoursInIC.contains(cc))) ){
									continue;
								}
								
								if (doContoursIntersect(cc, ic, currentZones)){
									Polygon cPoly2 = ic.getPolygon();
								
									for (int ccdP=0; ccdP < cPoly2.npoints; ccdP++){
										pnt2 = new Point2D.Double(cPoly2.xpoints[ccdP], cPoly2.ypoints[ccdP]);
										boolean pntInPoly2 = isPointInPolygon(cPoly2, pnt);
										boolean pnt2InPoly = isPointInPolygon(cPoly, pnt2);
										
										if (pntInPoly2 && pnt2InPoly){ 
											xyForceOnPnt = addxyRepulsionPntsOnDiffContours(xyForceOnPnt, pnt, pnt2, -1.0);
										
										} else if (!pntInPoly2 && pnt2InPoly){
											xyForceOnPnt = addxyRepulsionPntsOnDiffContours(xyForceOnPnt, pnt, pnt2, 2.0);
										
										} else if (pntInPoly2 && !pnt2InPoly){
											// do nothing
										} 
									}
								} else{
									
								}
							}
						}
					}
					
					if (repPntsNonICs){
						ArrayList<ConcreteContour> notIntersectingContours = contoursNotIntersectingWithContour_ForAllContours.get(cci);
						
						if (notIntersectingContours != null){
							for (ConcreteContour nic : notIntersectingContours){
								
								Zone commonZone = null;
								Zone smallestSplitZoneInCC = null;
								Zone smallestSplitZoneInNIC = null;
								if (doContoursIntersect(cc, nic, currentZones)){
									ArrayList<Zone> currZonesInCC = currZonesInContours[cci];
									ArrayList<Zone> currZonesInNIC = currZonesInContours[concreteContours.indexOf(nic)];
									
									String ccDesc = cc.getAbstractContour();
									String nicDesc = nic.getAbstractContour();
									
									//get common zone
									for (Zone zCC : currZonesInCC){
										if (currZonesInNIC.contains(zCC)){
											commonZone = zCC;
											break;
										}
									}
									if (commonZone != null){
										String commonZoneDesc = commonZone.getAbstractDescription();
										String splitZoneInCC_desc = commonZoneDesc.replace(nicDesc, "");
										String splitZoneInNIC_desc = commonZoneDesc.replace(ccDesc, "");
										
										double smallestSplitZoneInCCarea = 0;
										for (Zone zCC : currZonesInCC){
											if (zCC.getAbstractDescription().equals(splitZoneInCC_desc)){
												double zoneArea = zCC.computeTotalArea();

												if ((smallestSplitZoneInCC == null) || (smallestSplitZoneInCCarea > zoneArea)){
													smallestSplitZoneInCC = zCC;
													smallestSplitZoneInCCarea = zoneArea;
												}
												
											}
										}

										double smallestSplitZoneInNICarea = 0;
										for (Zone zNIC : currZonesInNIC){
											if (zNIC.getAbstractDescription().equals(splitZoneInNIC_desc)){
												double zoneArea = zNIC.computeTotalArea();
												if ((smallestSplitZoneInNIC == null) || (smallestSplitZoneInNICarea > zoneArea)){
													smallestSplitZoneInNIC = zNIC;
													smallestSplitZoneInNICarea = zoneArea;
												}
												
											}
										}
									}
								}
								
								
								
								Polygon cPoly2 = nic.getPolygon();
								
								for (int ccdP=0; ccdP < cPoly2.npoints; ccdP++){
									pnt2 = new Point2D.Double(cPoly2.xpoints[ccdP], cPoly2.ypoints[ccdP]);
									
									boolean pntIsValid = false;
									boolean pnt2IsValid = false;
									
									if ((commonZone != null) && (smallestSplitZoneInCC != null) && (smallestSplitZoneInNIC != null)){
										pntIsValid = (!smallestSplitZoneInCC.containsPoint(pnt)) || (!isPointInPolygon(cPoly2, pnt));
										pnt2IsValid = (!smallestSplitZoneInNIC.containsPoint(pnt2)) || (!isPointInPolygon(cPoly, pnt2));

									} else {
										pntIsValid = !isPointInPolygon(cPoly2, pnt);
										pnt2IsValid = !isPointInPolygon(cPoly, pnt2);
									}
									
									if ((pntIsValid && pnt2IsValid) || 
										(!pntIsValid && pnt2IsValid)){
									
										xyForceOnPnt = addxyRepulsionPntsOnDiffContours(xyForceOnPnt, pnt, pnt2, 1.0);
									} else if (!pntIsValid && !pnt2IsValid && ( (smallestSplitZoneInNIC == null) || (smallestSplitZoneInNIC.containsPoint(pnt2)) )) { 
										xyForceOnPnt = addxyAttractionPntsOnDiffContours2(xyForceOnPnt, pnt, pnt2, 1.0);
									} else if (pntIsValid && !pnt2IsValid) {
										//do nothing
									}									
								}
							}
						}
					}
					
					if (repOnContainingContours){
					//check whether this contour contains other contours and apply repulsive force accordingly
						ArrayList<ConcreteContour> contoursContained = contoursContainingContours.get(cc);
						
						if (contoursContained != null){
							for (ConcreteContour contourContained : contoursContained){
								
								Polygon outerPoly = cPoly;
								Polygon innerPoly = contourContained.getPolygon();
								Point2D.Double pntOnOuter = pnt;
								for (int ccdP=0; ccdP < innerPoly.npoints; ccdP++){
									Point2D.Double pntOnInner = new Point2D.Double(innerPoly.xpoints[ccdP], innerPoly.ypoints[ccdP]);
									
									boolean pntOnOuterIsValid = !isPointInPolygon(innerPoly, pntOnOuter);
									boolean pntOnInnerIsValid = isPointInPolygon(outerPoly, pntOnInner);
									
									if ((pntOnOuterIsValid && pntOnInnerIsValid) || 
										(!pntOnOuterIsValid && pntOnInnerIsValid)){
									
										xyForceOnPnt = addxyRepulsionPntsOnDiffContours(xyForceOnPnt, pntOnOuter, pntOnInner, 0.0);
									} else if (!pntOnOuterIsValid && !pntOnInnerIsValid) { 
									    xyForceOnPnt = addxyAttractionPntsOnDiffContours2(xyForceOnPnt, pntOnOuter, pntOnInner, 0.0);
									} else if (pntOnOuterIsValid && !pntOnInnerIsValid) {
										//do nothing
									} 
								}
							}
						}		
					}

					if (repOnContainedContours){
					//check whether this contour contains other contours and apply repulsive force accordingly
						ArrayList<ConcreteContour> contoursCCisContainedIn = contoursContainedInContours.get(cc);
						
						if (contoursCCisContainedIn != null){
							for (ConcreteContour contourCCisContainedIn : contoursCCisContainedIn){
								
								Polygon innerPoly = cPoly;
								Polygon outerPoly = contourCCisContainedIn.getPolygon();
								Point2D.Double pntOnInner = pnt;
								for (int ccdP=0; ccdP < outerPoly.npoints; ccdP++){
									Point2D.Double pntOnOuter = new Point2D.Double(outerPoly.xpoints[ccdP], outerPoly.ypoints[ccdP]);
									
									boolean pntOnOuterIsValid = !isPointInPolygon(innerPoly, pntOnOuter);
									boolean pntOnInnerIsValid = isPointInPolygon(outerPoly, pntOnInner);
									
									if ((pntOnOuterIsValid && pntOnInnerIsValid) ){
										xyForceOnPnt = addxyRepulsionPntsOnDiffContours(xyForceOnPnt, pntOnInner, pntOnOuter, 0.5);
									} else if ( (!pntOnOuterIsValid && !pntOnInnerIsValid) || 
											    (pntOnOuterIsValid && !pntOnInnerIsValid) ){
									    xyForceOnPnt = addxyAttractionPntsOnDiffContours2(xyForceOnPnt, pntOnInner, pntOnOuter, 0.5);
									} else if (!pntOnOuterIsValid && pntOnInnerIsValid) {
										//do nothing
									} 
								}
							}
						}		
					}
					
					
					
					
					//repulsion forces from points on other contours residing in this contour
					if (repAllPnts){
	 				    ArrayList<ConcreteContour> intersectingContours = intersectingContoursForAllContours.get(cci);
	   				    ArrayList<Point2D.Double> pntsOfICsInCC = getPointsOfContoursResidingInContour(cc, intersectingContours);
						for (Point2D.Double icPnt : pntsOfICsInCC){
							xyForceOnPnt = addxyRepulsionPntsOnDiffContours(xyForceOnPnt, pnt, icPnt, 0.0); 
						}					
					}
					

					//calculate attractive force 
					//    for every spring (i.e. edge) attached to the node
					a+=aInc;
					for (ContourLine cl : contourLines){
						Line2D.Double clPnts = cl.getLine();
						if ((clPnts.x1 == pnt.x) && (clPnts.y1 == pnt.y)){
							pnt2 = new Point2D.Double (clPnts.x2, clPnts.y2);
							xyForceOnPnt = addxyAttraction(xyForceOnPnt, pnt, pnt2, 0.0);
						} else if ((clPnts.x2 == pnt.x) && (clPnts.y2 == pnt.y)){
							pnt2 = new Point2D.Double (clPnts.x1, clPnts.y1);
							xyForceOnPnt = addxyAttraction(xyForceOnPnt, pnt, pnt2, 0.0);
						}
					}	
					a-=aInc;
						
					xyForceContourPoints.add(xyForceOnPnt);
					
				}
				
				
				ArrayList<Point2D.Double> xyForceContourLines = new ArrayList<Point2D.Double>(); 
				
				
				if (clToPntRep){
				
					ArrayList<ConcreteContour> intersectingContours = intersectingContoursForAllContours.get(cci);
						
					cc.setContourLines();
					ArrayList<ContourLine> cls = cc.getContourLines();
					
					ArrayList<ContourLine> icClsInContour = getContourLinesInOrIntsectingContour(cc, intersectingContours);
									
					ArrayList<Point2D.Double> pntsOfICsInCC = getPointsOfContoursResidingInContour(cc, intersectingContours);
				
					for (ContourLine cl : cls){
						Line2D.Double cll = cl.getLine();
						
						Point2D.Double lineMidPnt = getMidPnt(new Point2D.Double(cll.x1, cll.y1), new Point2D.Double(cll.x2, cll.y2));
	
						Point2D.Double xyForceOnMidPnt = new Point2D.Double(0,0);
						for (Point2D.Double icp : pntsOfICsInCC){
	
							Point2D.Double xyDistBetContourIntPnts = getxyAbsDistances (lineMidPnt, icp);
							Point2D.Double xyDist = xyDistBetContourIntPnts;
							double distance = getAbsHypotenuse(xyDistBetContourIntPnts);
							
							double fcr = DEFAULT_RD2; 
							if (distance <2) distance = 2;
							double totalForce = fcr / Math.pow(distance,2);
							
							Point2D.Double dir = getForceDirection (ForceType.r, lineMidPnt, icp);
							xyForceOnMidPnt.x +=  dir.x * (xyDist.x/(xyDist.x+xyDist.y)) * totalForce * DEFAULT_M; 
							xyForceOnMidPnt.y +=  dir.y * (xyDist.y/(xyDist.x+xyDist.y)) * totalForce * DEFAULT_M;
						}
						xyForceContourLines.add(xyForceOnMidPnt);
					}
				}

				xyForceOnContoursLines.add(xyForceContourLines);
				xyForceOnContoursPoints.add(xyForceContourPoints);
				
				
				if (zoneAreaAdap2){
					// Get zones in current contour which are shared with other contours
				    ArrayList<Zone> zonesAllContours = getZones(concreteContours,false);
				    ArrayList<Zone> zonesRelatedToCurrentContour = getZonesRelatedToContour(cc, zonesAllContours);
				    ArrayList<String> zonesSharedAbsDesc = new ArrayList<String>();
				    ArrayList<String> zonesOutAbsDesc = new ArrayList<String>();
					for(Zone z: zonesRelatedToCurrentContour){
						if (z.getAbstractDescription().length()>1)
							zonesSharedAbsDesc.add(z.getAbstractDescription());
						else
							zonesOutAbsDesc.add(z.getAbstractDescription());
					}
					Collections.sort(zonesSharedAbsDesc, new Comparator<String>(){
						public int compare (String zAbsDesc1, String zAbsDesc2) {
							return (zAbsDesc1.length()-zAbsDesc2.length());
						}
					});
					
					ArrayList<Zone> neighbouringZonesOfICs = getNeighbouringZonesOfIntersectingContours(cc, zonesAllContours, concreteContours);
					
					// Consider every shared zone
		 			for (String zAbsDesc : zonesSharedAbsDesc){
	
						Zone currentZone = Zone.getZoneFromAbstractDescription(zAbsDesc, zonesAllContours);
						if (currentZone == null){continue;}
		 				ArrayList<ConcreteContour> otherContoursOfZone = getOtherContoursOfZone(cc, currentZone, concreteContours);
						ArrayList<Zone> neighbouringZonesOfICsWithZone = removeZonesNotInAnyContours(otherContoursOfZone, neighbouringZonesOfICs);
						
						double fc = DEFAULT_ZONEAREA_ADAP;
						double totalZoneForceOnContour = fc * (avgZoneAreasForNSets[currentZone.getAbstractDescription().length()-1] - currentZone.computeTotalArea());
	
						
						// calculate x y force movement of contour
						if ((computeContourCentroid(cc)!=null)&&(currentZone.computeCentroid()!=null)){
							
						Point2D.Double dir = getForceDirection (ForceType.r, currentZone.computeCentroid(), computeContourCentroid(cc));
						Point2D.Double xyDistance = getxyAbsDistances (currentZone.computeCentroid(), computeContourCentroid(cc));
						
	
						xyForceOnContour.x += dir.x * (xyDistance.x/(xyDistance.x+xyDistance.y)) * totalZoneForceOnContour;
						xyForceOnContour.y += dir.y * (xyDistance.y/(xyDistance.x+xyDistance.y)) * totalZoneForceOnContour;
						}
		 			}
		 			double currentContourArea = computeContourArea(cc);
		 			double idealContourArea = 0;
		 			for(Zone z: zonesRelatedToCurrentContour){
		 				idealContourArea += avgZoneAreasForNSets[z.getAbstractDescription().length()-1];
		 			}
		 			
		 			if ((idealContourArea-currentContourArea)>0){
		 				cc.setPolygon(transformPolygon(cc.getPolygon(), (0.005*((idealContourArea-currentContourArea)/currentContourArea))+1, 0.0));
		 				cc.setContourLines();
		 			}
	 			
				}
				
			
				if (attICs){
				ArrayList<ConcreteContour> currentICs = getIntersectingContours(cc, concreteContours);
				ArrayList<ConcreteContour> contoursContained = contoursContainingContours.get(cc);
	 			for (ConcreteContour ic : currentICs){
	 				
	 				if ((contoursContained!=null)&&(contoursContained.contains(ic))){continue;} 
	 				
					Point2D.Double ccCentroid = computeContourCentroid(cc);
					Point2D.Double icCentroid = computeContourCentroid(ic);					

					Point2D.Double xyDistBetCCcentICcent = getxyAbsDistances (ccCentroid, icCentroid);
					double distBetCCcentICcent = getAbsHypotenuse(xyDistBetCCcentICcent);
					
					MyLine centToCentLine = new MyLine (ccCentroid, icCentroid);
					Line2D.Double centroidToCentroidLine = null;
					if (xyDistBetCCcentICcent.x <0){
						centroidToCentroidLine = new Line2D.Double(new Point2D.Double (ccCentroid.x-distBetCCcentICcent, (centToCentLine.getGradient()*(ccCentroid.x-distBetCCcentICcent))+centToCentLine.getYIntercept()), new Point2D.Double (icCentroid.x+distBetCCcentICcent, (centToCentLine.getGradient()*(icCentroid.x+distBetCCcentICcent))+centToCentLine.getYIntercept()));
					}else{
						centroidToCentroidLine = new Line2D.Double(new Point2D.Double (icCentroid.x-distBetCCcentICcent, (centToCentLine.getGradient()*(icCentroid.x-distBetCCcentICcent))+centToCentLine.getYIntercept()), new Point2D.Double (ccCentroid.x+distBetCCcentICcent, (centToCentLine.getGradient()*(ccCentroid.x+distBetCCcentICcent))+centToCentLine.getYIntercept()));						
					}
					
						
					if(getLineIntersectionPointsWithContour(cc, new MyLine (centroidToCentroidLine)).size()<1){continue;}
					if(getLineIntersectionPointsWithContour(ic, new MyLine (centroidToCentroidLine)).size()<1){continue;}
						
					Point2D.Double ccIntPnt= getLineIntersectionPointsWithContour(cc, new MyLine (centroidToCentroidLine)).get(0);
					Point2D.Double icIntPnt= getLineIntersectionPointsWithContour(ic, new MyLine (centroidToCentroidLine)).get(0);
					
					Point2D.Double xyDistBetCCintpntICintpnt = getxyAbsDistances (ccIntPnt, icIntPnt);
					double distBetCCintpntICintpnt = getAbsHypotenuse(xyDistBetCCintpntICintpnt);
					
					Point2D.Double xyDistBetCCcentICintpnt = getxyAbsDistances (ccCentroid, icIntPnt);
					double distBetCCcentICintpnt = getAbsHypotenuse(xyDistBetCCcentICintpnt);

					double fcA = 0.05; 
					double totalForce = 0;
					Point2D.Double dir = new Point2D.Double(0,0);
					
					totalForce = fcA * distBetCCcentICcent; 
					dir = getForceDirection (ForceType.a, ccCentroid, icCentroid);
					
					xyForceOnContour.x +=  dir.x * (xyDistBetCCcentICintpnt.x/(xyDistBetCCcentICintpnt.x+xyDistBetCCcentICintpnt.y)) * totalForce * DEFAULT_M;
					xyForceOnContour.y +=  dir.y * (xyDistBetCCcentICintpnt.y/(xyDistBetCCcentICintpnt.x+xyDistBetCCcentICintpnt.y)) * totalForce * DEFAULT_M;
					
				}
				}
				if (repNonICs && (contoursNotIntersectingWithContour_ForAllContours.size() != 0)){ 
					ArrayList<ConcreteContour> contoursNotIntersectingWithCC = contoursNotIntersectingWithContour_ForAllContours.get(cci);
					for (ConcreteContour nicWithCC : contoursNotIntersectingWithCC){
						
						Point2D.Double ccCentroid = computeContourCentroid(cc);
						Point2D.Double ciwicCentroid = computeContourCentroid(nicWithCC);
						
	
						double distance=0;
						Point2D.Double xyDist = new Point2D.Double(0,0);
						Line2D.Double centroidToCentroidLine = new Line2D.Double (ccCentroid, ciwicCentroid);
						
						ArrayList <Point2D.Double> ccIntPnts = getLineIntersectionPointsWithContour(cc, new MyLine(centroidToCentroidLine));
						ArrayList <Point2D.Double> ciwicIntPnts = getLineIntersectionPointsWithContour(nicWithCC, new MyLine(centroidToCentroidLine));
						if(ccIntPnts.size()<1){continue;}
						if(ciwicIntPnts.size()<1){continue;}
							
						Point2D.Double ccIntPnt= ccIntPnts.get(0);
						Point2D.Double ciwicIntPnt= ciwicIntPnts.get(0);
							
						Point2D.Double xyDistBetContourIntPnts = getxyAbsDistances (ccIntPnt, ciwicIntPnt);
						xyDist = xyDistBetContourIntPnts;
						distance = getAbsHypotenuse(xyDistBetContourIntPnts);
						
						ArrayList<ConcreteContour> currentlyICsWithCC = getIntersectingContours(cc, concreteContours);
						
						double fcr = 200;
						Point2D.Double dir = new Point2D.Double(0,0);
						double totalForce = 0;
						if (!currentlyICsWithCC.contains(nicWithCC)){
							totalForce = fcr / Math.pow(distance, 1.0);
							dir = getForceDirection (ForceType.r, ccCentroid, ciwicCentroid);
						} else { // structure broken
							totalForce = 0.1 * Math.pow(distance+5, 1.0);
							dir = getForceDirection (ForceType.r, ccCentroid, ciwicCentroid); 
						}
						
						xyForceOnContour.x +=  dir.x * (xyDist.x/(xyDist.x+xyDist.y)) * totalForce * DEFAULT_M;
						xyForceOnContour.y +=  dir.y * (xyDist.y/(xyDist.x+xyDist.y)) * totalForce * DEFAULT_M;
					}
				}

				xyForceOnContours.add(xyForceOnContour);
				
				cci++;
			}
			
			ArrayList<ConcreteContour> origCCs = new ArrayList<ConcreteContour>();
			for (ConcreteContour cc : concreteContours){
				origCCs.add(cc.clone());
			}

			
			//
			int cii=0;
			for (Point2D.Double xyForceOnContour : xyForceOnContours){
				ConcreteContour cc = concreteContours.get(cii);
				Point2D.Double xyAllForceOnContour = new Point2D.Double();
				xyAllForceOnContour.x = xyForceOnContour.x + xyOtherForceOnContours[cii].x;
				xyAllForceOnContour.y = xyForceOnContour.y + xyOtherForceOnContours[cii].y;
				moveContour(cc, xyAllForceOnContour);
				cc.setPolygon(cc.getPolygon());
				cc.setContourLines();
				cc.resetArea();
				cii++;
			}	
			
			if (clToPntRep){
			int cCount =0;
			for(ConcreteContour cc : concreteContours){
				
				ArrayList<Point2D.Double> xyForceContourLines = xyForceOnContoursLines.get(cCount);
			
				ArrayList<ContourLine> cls = cc.getContourLines();
				ArrayList<ContourLine> newCls = new ArrayList<ContourLine>();
				int clsCount =0;
				Polygon cPoly = cc.getPolygon();
				ArrayList<int[]> clsPntIndexMatch = new ArrayList<int[]>();
				for (ContourLine cl: cls){
					Line2D.Double cll = cl.getLine();
					
					boolean clPnt1Found = false;
					boolean clPnt2Found = false;
					int[] clPntIndexMatch = new int[2];
					for (int p =0; p < cPoly.npoints; p++){
						if ((!clPnt1Found) && (cll.x1 == cPoly.xpoints[p]) && (cll.y1 == cPoly.ypoints[p])){
							clPntIndexMatch[0] = p;
							clPnt1Found=true;
						} else if ((!clPnt2Found) && (cll.x2 == cPoly.xpoints[p]) && (cll.y2 == cPoly.ypoints[p])){
							clPntIndexMatch[1] = p;
							clPnt2Found=true;
						}
						if (clPnt1Found && clPnt2Found){
							clsPntIndexMatch.add(clPntIndexMatch);
							break;
						}
					}	
				}
				
				
				int clCount=0;
				for (ContourLine cl : cls){
					
					Line2D.Double cll = cl.getLine();
					Point2D.Double pnt1 = new Point2D.Double(cll.x1, cll.y1);
					Point2D.Double pnt2 = new Point2D.Double(cll.x2, cll.y2);
					Point2D.Double xyForceContourLine = xyForceContourLines.get(clsCount);
					Point2D.Double newPnt1Pos = getNewPntPos(xyForceContourLine, pnt1);
					Point2D.Double newPnt2Pos = getNewPntPos(xyForceContourLine, pnt2);
					
					int[] clPntIndexMatch = clsPntIndexMatch.get(clCount);
					cPoly.xpoints[clPntIndexMatch[0]] = (int) (Math.round (newPnt1Pos.x));
					cPoly.ypoints[clPntIndexMatch[0]] = (int) (Math.round (newPnt1Pos.y));	
					cPoly.xpoints[clPntIndexMatch[1]] = (int) (Math.round (newPnt2Pos.x));
					cPoly.ypoints[clPntIndexMatch[1]] = (int) (Math.round (newPnt2Pos.y));	
					
					clCount++;
				}
			
				cc.setPolygon(cPoly);
				cc.setContourLines();
				cCount++;
			}
			}
			
			
			
			for (int ci = 0; ci < xyForceOnContoursPoints.size(); ci++){
				ConcreteContour cc = concreteContours.get(ci);
		
				Polygon cPoly = cc.getPolygon();
				ArrayList<Point2D.Double> xyForceContourPoints = xyForceOnContoursPoints.get(ci);
				
				for (int p =0; p < xyForceContourPoints.size(); p++){
					Point2D.Double pnt = new Point2D.Double(cPoly.xpoints[p], cPoly.ypoints[p]);
					Point2D.Double newPntPost = getNewPntPos(xyForceContourPoints.get(p),pnt);
					
					cPoly.xpoints[p] = (int) (Math.round (newPntPost.x));
					cPoly.ypoints[p] = (int) (Math.round (newPntPost.y));	
				}
				
				cc.setPolygon(cPoly);
				cc.setContourLines();
				cc.resetArea();
			}	
			
			if (zoneAreaAdap && (contoursEnlargeFactor != null)){
				cci=0;
				for (ConcreteContour cc : concreteContours){
					if (contoursEnlargeFactor[cci] > 0){
						cc.setPolygon(transformPolygon(cc.getPolygon(), contoursEnlargeFactor[cci] ,0.0)); 
					
						cc.resetArea();
					}
					cci++;
			   }
			}

			
			if(causeOnlyPosChange){ 
				String currAbstractDesc = ConcreteContour.generateAbstractDiagramFromList(concreteContours);
				if (!initialAbstractDesc.equals(currAbstractDesc)){
					int occI =0;
					for (ConcreteContour occ : origCCs){
						ConcreteContour cc = concreteContours.get(occI);
						cc.setPolygon(occ.getPolygon());
						cc.setContourLines();
						occI++;
					}
				}
			}
			
			centreOnScreen(concreteContours, simpleDiagramPanel.getWinWidth(), simpleDiagramPanel.getWinHeight());
			simpleDiagramPanel.updateDisplay();
			
			concreteContours = equalizePointsOnContoursAccordingToDist(concreteContours, 80); 

		}
		simpleDiagramPanel.getStopWatch().stop();
		
		centreOnScreen(concreteContours, simpleDiagramPanel.getWinWidth(), simpleDiagramPanel.getWinHeight());
		simpleDiagramPanel.recordAsFinalLayout();
		simpleDiagramPanel.updateDisplay();
		
		String finalAbstractDesc = ConcreteContour.generateAbstractDiagramFromList(concreteContours);
		finalAbstractDesc = ConcreteContour.generateAbstractDiagramFromList(concreteContours);
		fitToScreen (concreteContours, simpleDiagramPanel.getWinWidth(), simpleDiagramPanel.getWinHeight(), true);		
	}

	
	public static void fitToScreen (ArrayList<ConcreteContour> ccs, int winWidth, int winHeight, boolean scaleOnlyIfTooBigOrTooSmall){
	
		int x1 = 100; 
		int y1 = 30;
		int x2 = (int)Math.round(winWidth  * 0.95);
		int y2 = (int)Math.round(winHeight * 0.85) - 100; //(-100 because of the bottom bar with details)
	
		int width = ConcreteDiagram.findContoursWidth(ccs);
		int height = ConcreteDiagram.findContoursHeight(ccs);
		int rectX1 = x1;
		int rectX2 = x2;
		if(rectX1 > rectX2) {
			rectX1 = x2;
			rectX2 = x1;
		}
		int rectY1 = y1;
		int rectY2 = y2;
		if(rectY1 > rectY2) {
			rectY1 = y2;
			rectY2 = y1;
		}
		int requiredX = rectX2-rectX1;
		int requiredY = rectY2-rectY1;
		
		int centreX = rectX1 + requiredX/2;
		int centreY = rectY1 + requiredY/2;
		
		
		//this is used to enlarge diagram to fit precisely in the screen
		double xRatio = requiredX/(double)width;
		double yRatio = requiredY/(double)height;
		// scale to the ratio that will not take the graph over the limits
		double scaleFactor = xRatio;
		if(yRatio < xRatio) {
			scaleFactor = yRatio;
		}
		
		boolean tooBig = (scaleFactor < 1);
		boolean tooSmall = ((width < (0.2*winWidth)) && (height < (0.2*winHeight))); 
		
		if (scaleOnlyIfTooBigOrTooSmall){
			if (tooBig || tooSmall){    //scale only if it does not fit in screen (scaleFactor < 1) or it is too small (ie the width and the height of the diagram are both less then a third of those of the window)
				if (tooSmall){
					scaleFactor /= 2;
				}
				ConcreteDiagram.scaleContours(ccs, scaleFactor);
				ConcreteDiagram.centreContoursOnPoint(ccs, centreX, centreY);
			}
		}
	}

	
	
	public static void centreOnScreen(ArrayList<ConcreteContour> ccs, int winWidth, int winHeight){

		int x1 = 0;
		int y1 = 0;
		int x2 = (int)Math.round(winWidth  * 0.95);
		int y2 = (int)Math.round(winHeight * 0.85);
	
		int width = ConcreteDiagram.findContoursWidth(ccs);
		int height = ConcreteDiagram.findContoursHeight(ccs);
		int rectX1 = x1;
		int rectX2 = x2;
		if(rectX1 > rectX2) {
			rectX1 = x2;
			rectX2 = x1;
		}
		int rectY1 = y1;
		int rectY2 = y2;
		if(rectY1 > rectY2) {
			rectY1 = y2;
			rectY2 = y1;
		}
		int requiredX = rectX2-rectX1;
		int requiredY = rectY2-rectY1;
		
		int centreX = rectX1 + requiredX/2;
		int centreY = rectY1 + requiredY/2;
		
		ConcreteDiagram.centreContoursOnPoint(ccs, centreX, centreY);
	}
	
	
	
	public static Point2D.Double convertToPoint2D_Double (Point pnt){
		return (new Point2D.Double (new Double(pnt.x), new Double(pnt.y)));
	}
	public static Point convertToPoint (Point2D.Double pnt){
		return (new Point((int) Math.round(pnt.x), (int) Math.round(pnt.y)));
	}
	
	
	
	private Point2D.Double getLineIntersectionPoint (Line2D.Double line1, Line2D.Double line2){
		
		Point2D.Double intersectionPoint = null;
		
		Point2D.Double distanceL1 = new Point2D.Double(0,0);
		Point2D.Double distanceL2 = new Point2D.Double(0,0);
	
		double cL1, cL2, denom;
		
		distanceL1.x = line1.getX1()-line1.getX2();
		distanceL1.y = line1.getY2()-line1.getY1();
		cL1 = line1.getX2()*line1.getY1()-line1.getX1()*line1.getY2(); 
		// a1x + b1y + c1 = 0 line1 eq
		
		distanceL2.x = line2.getX1()-line2.getX2();
		distanceL2.y = line2.getY2()-line2.getY1();
		cL2 = line2.getX2()*line2.getY1()-line2.getX1()*line2.getY2(); 
		// a2x + b2y + c2 = 0 line2 eq
		
		denom = distanceL1.y*distanceL2.x - distanceL2.y*distanceL1.x;
		if(denom != 0){
			intersectionPoint = new Point2D.Double((distanceL1.x*cL2 - distanceL2.x*cL1)/denom,(distanceL2.y*cL1 - distanceL1.y*cL2)/denom);
		} else {
			// lines are parallel
		}
		
		return intersectionPoint;
	}
	
	
	private Polygon transformPolygon(Polygon poly, double scaleFactor, double rotation) {
		
		Point2D.Double originalPolygonCentroid = convertToPoint2D_Double(Utilities.findMiddlePointInsidePolygon(poly));
		
		Polygon transformedPolygon = new Polygon(new int[poly.npoints], new int[poly.npoints], poly.npoints);
		
		double costheta = Math.cos(rotation);
		double sintheta = Math.sin(rotation);
		double rx, ry;
		
		for (int p=0; p < poly.npoints; p++){
			//rotate, scale, translate
			rx = (poly.xpoints[p] * costheta) - (poly.ypoints[p] * sintheta);
			ry = (poly.xpoints[p] * sintheta) + (poly.ypoints[p] * costheta);
			
			transformedPolygon.xpoints[p] = (int)Math.round((rx *scaleFactor));
			transformedPolygon.ypoints[p] = (int)Math.round((ry *scaleFactor));
	    }
		
	    return transformedPolygon;
	}
	
	
	private void changePolygonCentroid (Polygon poly, Point2D.Double newCentroid){
		
		Point2D.Double oldCentroid = convertToPoint2D_Double(Utilities.findMiddlePointInsidePolygon(poly));
		Point2D.Double xyPlacement = getxyDistances(oldCentroid, newCentroid);
		
		for (int p =0; p<poly.npoints; p++){
			poly.xpoints[p] +=  xyPlacement.x;
			poly.ypoints[p] +=  xyPlacement.y;
		}
	}
	
	
	private void changeContourCentroid (ConcreteContour contour, Point2D.Double newCentroid){
		
		Point2D.Double oldCentroid = computeContourCentroid(contour);
		Point2D.Double xyPlacement = getxyDistances(newCentroid, oldCentroid);
		
		moveContour (contour, xyPlacement);
	}
	
	
	private void moveContour(ConcreteContour c, Point2D.Double xyPlacement){
		ArrayList<ConcreteContour> contoursToMove = new ArrayList<ConcreteContour>();
		contoursToMove.add(c);
		ConcreteDiagram.moveContours(contoursToMove, (int)Math.round(xyPlacement.x), (int)Math.round(xyPlacement.y));
	}
	
	
	
	// Methods and Functionalities invoked and used by the Layout method
	
	// Equalise the number of points each contour has
	//      this can potentially be set as a public static method for other classes to invoke as a useful utility/functionality
	private ArrayList<ConcreteContour> equalizePointsOnContours (ArrayList<ConcreteContour> ccs){
		
		//  Find contour with max number of points 
		int maxPntsOnContour = 0;
		for (ConcreteContour cc : ccs){
			Polygon cPoly = cc.getPolygon();
			int cpolyPntsCount = cPoly.npoints;
			if (cpolyPntsCount > maxPntsOnContour)
				maxPntsOnContour = cpolyPntsCount;
		}
		
		maxPntsOnContour += ccs.size()+1;
		
		//  Add required points to contours
		//      by just splitting up the current contour lines
		for (ConcreteContour cc : ccs){
			Polygon cPoly = cc.getPolygon();
			
			int currentPntCount = cPoly.npoints;
			int pntsToAddCount = maxPntsOnContour - currentPntCount;
			if (pntsToAddCount == 0)
				continue;
			int finalPntCount = currentPntCount + pntsToAddCount;
			
			ArrayList<ContourLine> contourlines = cc.getContourLines();
			int pntsAddedCount = 0;
			int cl_i = 0;
			Polygon npoly = new Polygon();
			
			do{
				if (cl_i == contourlines.size()){
					cc.setPolygon(npoly);
					cc.setContourLines();
					cl_i = 0;
					npoly = new Polygon();
				}
				contourlines = cc.getContourLines();

				ContourLine cl = contourlines.get(cl_i);
				Line2D.Double cl_line = cl.getLine(); 

				Point2D.Double newPnt = getMidPnt(new Point2D.Double (cl_line.x1, cl_line.y1), new Point2D.Double (cl_line.x2, cl_line.y2));
				npoly.addPoint((int)Math.round(cl_line.x1), (int)Math.round(cl_line.y1));
				npoly.addPoint((int)Math.round(newPnt.x), (int)Math.round(newPnt.y)); 
 
				pntsAddedCount++;
				cl_i++;
				
			} while (pntsAddedCount < pntsToAddCount);
			
			while (npoly.npoints < finalPntCount){
				ContourLine cl = contourlines.get(cl_i);
				Line2D.Double cl_line = cl.getLine(); 

				npoly.addPoint((int)Math.round(cl_line.x1), (int)Math.round(cl_line.y1));
				
				cl_i++;
			}
			cc.setPolygon(npoly);
			cc.setContourLines();
		}
		
		return ccs;
	}
	
	
	
	private ArrayList<ConcreteContour> equalizePointsOnContoursAccordingToDist (ArrayList<ConcreteContour> ccs, double maxClDist){
		//  Find contour with max number of points 
		if (maxClDist <= 0){return null;}
		for (ConcreteContour cc : ccs){
			ArrayList<ContourLine> cls = cc.getContourLines();
			Polygon npoly = new Polygon();
			for (ContourLine cl : cls){
				Line2D.Double clLine = cl.getLine();
				Point2D.Double clpnt1 = new Point2D.Double(clLine.x1, clLine.y1);
				Point2D.Double clpnt2 = new Point2D.Double(clLine.x2, clLine.y2);
				double clLen = getAbsDistance(clpnt1, clpnt2);
				
				int noOfPointsToAdd = (int)(clLen / maxClDist);
				
				if (noOfPointsToAdd >0 ){
					
					Point2D.Double xyDist = getxyDistances(clpnt2, clpnt1);
					double angCloseToClPnt1 = Math.asin(Math.abs(xyDist.y)/clLen);
					double incX = Math.signum(xyDist.x) * Math.round(maxClDist*Math.cos(angCloseToClPnt1));
					double incY = Math.signum(xyDist.y) * Math.round(maxClDist*Math.sin(angCloseToClPnt1));
					
					npoly.addPoint((int)Math.round(clLine.x1), (int)Math.round(clLine.y1));
					double nextX = clpnt1.x;
					double nextY = clpnt1.y;
					for (int i =0; i<noOfPointsToAdd;i++){
						nextX += incX;
						nextY += incY;
						npoly.addPoint((int)nextX, (int)nextY);
					}

				}else{
					npoly.addPoint((int)Math.round(clLine.x1), (int)Math.round(clLine.y1));
				}
			}
			cc.setPolygon(npoly);
			cc.setContourLines();
		}
		return ccs;
	}
	
	
	
	private ArrayList<Zone> getZones(ArrayList<ConcreteContour> ccs){
		return getZones (ccs, true);
	}
	
	private ArrayList<Zone> getZones(ArrayList<ConcreteContour> ccs, boolean includeEmptyZones){
		
		HashMap<String, Area> zoneAreas = ConcreteContour.generateZoneAreas(ccs);
		Iterator itr = zoneAreas.entrySet().iterator();
		
		ArrayList<Zone> zones = new ArrayList<Zone>();
		
		while (itr.hasNext()){
			Entry<String,Area> strAreaPair = (Entry<String,Area>)itr.next();
			ArrayList<Polygon> zPolys = (ConcreteContour.polygonsFromArea(strAreaPair.getValue()));
			zones.add(new Zone(strAreaPair.getKey(), strAreaPair.getValue(), zPolys));
		}
		
		if (!includeEmptyZones){
			zones = Zone.removeEmptyZones(zones);
		}
		
		return zones;
	}
	
	
	
	private HashMap<String, ArrayList<Zone>> getZonesForContourPoints (ArrayList<ConcreteContour> ccs, boolean includeEmptyZones){
		ArrayList<Zone> zones = getZones(ccs, includeEmptyZones);
		return getZonesForContourPoints(ccs, zones);
	}
	
	private HashMap<String, ArrayList<Zone>> getZonesForContourPoints (ArrayList<ConcreteContour> ccs, ArrayList<Zone> zones, boolean includeEmptyZones){
		//this method is not really necessary -> if they want to remove the empty zones, they can do so when generating the zones
		if (!includeEmptyZones){
			zones = Zone.removeEmptyZones(zones);
		}
		return getZonesForContourPoints(ccs, zones);
	}
	
	private HashMap<String, ArrayList<Zone>> getZonesForContourPoints (ArrayList<ConcreteContour> ccs, ArrayList<Zone> zones){
		double pntPosVar = .5;
		double sqPntPosVarWidth = pntPosVar*2;
		
		HashMap<String, ArrayList<Zone>> zonesForPnts = new HashMap<String, ArrayList<Zone>>();
	
		int ci =0;
		for (ConcreteContour cc : ccs){
			Polygon cPoly = cc.getPolygon();
			
			for (int p=0; p < cPoly.npoints; p++){

				String pntID = Integer.toString(ci) + "|" + Integer.toString(p);
				
				int pi=0;
				for (Zone z : zones){
					ArrayList<Polygon> zPolys = z.getPolygons();
			
					for (Polygon zPoly : zPolys){
						
						if ( (zPoly.contains(cPoly.xpoints[p], cPoly.ypoints[p])) || 
							 (zPoly.intersects(cPoly.xpoints[p]-pntPosVar, cPoly.ypoints[p]-pntPosVar, sqPntPosVarWidth, sqPntPosVarWidth))){
							
							if (zonesForPnts.containsKey(pntID)){
								(zonesForPnts.get(pntID)).add(z);
							} else {
								ArrayList<Zone> zonesForPnt = new ArrayList<Zone>();
								zonesForPnt.add(z);
								zonesForPnts.put(pntID, zonesForPnt);
							}
						}
					}
					pi++;
				}
			}	
			ci++;
		}
		return zonesForPnts;
	}
	
	
	
	private ArrayList<Zone> getCommonZonesOfIntersectingContours (ConcreteContour cc1, ConcreteContour cc2, ArrayList<Zone> zones){
		
		ArrayList<Zone> zonesOfIntersectingContours = new ArrayList<Zone>();
		
		for (Zone z : zones){
			String zoneAbsDesc = z.getAbstractDescription();
			if (zoneAbsDesc.contains(cc1.getAbstractContour()) && zoneAbsDesc.contains(cc1.getAbstractContour())){
				zonesOfIntersectingContours.add(z);
			}
		}
		return zones;
	}

	
	
	private boolean doContoursIntersect (ConcreteContour cc1, ConcreteContour cc2, ArrayList<Zone> zones){
		
		for (Zone z : zones){
			String zoneAbsDesc = z.getAbstractDescription();
			if (zoneAbsDesc.contains(cc1.getAbstractContour()) && zoneAbsDesc.contains(cc2.getAbstractContour())){
				return true;
			}
		}
		return false;
	}
	
	
	private ArrayList<ConcreteContour> getIntersectingContours(ConcreteContour cc, ArrayList<ConcreteContour> ccs){
		return getIntersectingContours (cc, ccs, getZones(ccs, false));	
	}
	
	private ArrayList<ConcreteContour> getIntersectingContours(ConcreteContour cc, ArrayList<ConcreteContour> ccs, ArrayList<Zone> zones){
		
		ArrayList<ConcreteContour> intersectingContours = new ArrayList<ConcreteContour>();
		
		for (ConcreteContour c2 : ccs){
			if (cc == c2){continue;}
					
			if (doContoursIntersect(cc, c2, zones)){
				intersectingContours.add(c2);
			}
		}
		return intersectingContours;
		
	}
	
	
	private ArrayList<ContourLine> getContourLinesInOrIntsectingContour (ConcreteContour contour, ArrayList<ConcreteContour> otherContours){
		ArrayList<ContourLine> clsInContour = new ArrayList<ContourLine>();
		
		for (ConcreteContour oc : otherContours){
			Polygon ocpoly = oc.getPolygon();
			
			ArrayList<ContourLine> cls = oc.getContourLines();
			for (ContourLine cl : cls){
				Line2D.Double l = cl.getLine();
				Point2D.Double pnt1 = new Point2D.Double(l.x1, l.y1);
				Point2D.Double pnt2 = new Point2D.Double(l.x2, l.y2);
				if (isLineInPolygon(ocpoly, cl.getLine()) ||
					isPointInPolygon(ocpoly, pnt1)||isPointInPolygon(ocpoly, pnt2)){
					clsInContour.add(cl);
				}
			}
		}
		return clsInContour;
	}

	
	
	private ArrayList<Point2D.Double> getPointsOfContoursResidingInContour (ConcreteContour contour, ArrayList<ConcreteContour> otherContours){
		double pntPosVar = 0.5;
		double sqPntPosVarWidth = pntPosVar*2;
		
		ArrayList<Point2D.Double> pntsOfContoursResidingInContour = new ArrayList<Point2D.Double>();
		Polygon cpoly = contour.getPolygon();
		
		for (ConcreteContour oc : otherContours){
			Polygon ocpoly = oc.getPolygon();
			
			for (int p =0; p<ocpoly.npoints; p++){
				if ( (cpoly.contains(ocpoly.xpoints[p], ocpoly.ypoints[p])) || 
					 (cpoly.intersects(ocpoly.xpoints[p]-pntPosVar, ocpoly.ypoints[p]-pntPosVar, sqPntPosVarWidth, sqPntPosVarWidth))){
					pntsOfContoursResidingInContour.add(convertToPoint2D_Double(new Point (ocpoly.xpoints[p], ocpoly.ypoints[p])));
				}
			}		
		}
		
		return pntsOfContoursResidingInContour;
	}
	
	
	
	private ArrayList<Point2D.Double> getSomePointsInContourOfICs (ConcreteContour contour, ArrayList<ConcreteContour> otherContours){
		
		ArrayList<Point2D.Double> somePointsInContourOfICs = new ArrayList<Point2D.Double>();
		ArrayList<Line2D.Double> linesInContour = new ArrayList<Line2D.Double>();
		
		Polygon cpoly = contour.getPolygon();
		
		for (ConcreteContour oc : otherContours){
			Polygon ocpoly = oc.getPolygon();
			ArrayList<Line2D.Double> ocLines = getPolygonLines(ocpoly);
			
			for (Line2D.Double ocLine : ocLines){
				Point2D.Double ocLinePnt1 = new Point2D.Double(ocLine.x1, ocLine.y1);
				Point2D.Double ocLinePnt2 = new Point2D.Double(ocLine.x2, ocLine.y2);
				
				if (isLineInPolygon(cpoly, ocLine)){
					linesInContour.add(ocLine);
				}else{
					ArrayList<Point2D.Double> intPnts = getLineIntersectionPointsWithContour(contour, new MyLine (ocLine.x1, ocLine.y1, ocLine.x2, ocLine.y2));			
					if (intPnts.size() >0){
						linesInContour.add(ocLine);
					}
				}
			}
		}
		
		for (Line2D.Double l : linesInContour){
			Point2D.Double pnt1 = new Point2D.Double (l.x1,l.y1);
			Point2D.Double pnt2 = new Point2D.Double (l.x2,l.y2);
			double lineLength = getAbsHypotenuse(getxyAbsDistances(pnt1, pnt2));
			int npnts = (int)Math.round(lineLength/4);
			somePointsInContourOfICs.add(pnt1);
			somePointsInContourOfICs.add(pnt2);
		}
		
		return somePointsInContourOfICs;
	}

	
	
	public static boolean isPointInPolygon(Polygon poly, Point2D.Double pnt){
		double pntPosVar = 0.5;
		double sqPntPosVarWidth = pntPosVar*2;
		
		return ( poly.contains(pnt.x, pnt.y) || 
			     poly.intersects(pnt.x-pntPosVar, pnt.y-pntPosVar, sqPntPosVarWidth, sqPntPosVarWidth) );
	}
	
	
	
	private boolean isLineInPolygon (Polygon poly, Line2D.Double line){
		Point2D.Double pnt1 = new Point2D.Double(line.x1, line.y1);
		Point2D.Double pnt2 = new Point2D.Double(line.x2, line.y2);
		
		return ( isPointInPolygon(poly,pnt1) && isPointInPolygon(poly,pnt2) );
	}
	
	
	
	private ArrayList<Line2D.Double> getPolygonLines (Polygon poly){
		ArrayList<Line2D.Double> polyLines = new ArrayList<Line2D.Double>();
		
		for (int i = 1; i< poly.npoints; i++){
			polyLines.add(new Line2D.Double(poly.xpoints[i-1], poly.ypoints[i-1], poly.xpoints[i], poly.ypoints[i]));
		}
		polyLines.add(new Line2D.Double(poly.xpoints[poly.npoints-1], poly.ypoints[poly.npoints-1], poly.xpoints[0], poly.ypoints[0]));
		
		return polyLines;
	}
	
	
	
	private boolean isContourLineInContour(ContourLine cl, ConcreteContour c){
		Polygon cpoly = c.getPolygon();
		Line2D.Double clLine = cl.getLine();
		Point2D.Double clPnt1 = new Point2D.Double (clLine.x1, clLine.y1);
		Point2D.Double clPnt2 = new Point2D.Double (clLine.x2, clLine.y2);
		return (isPointInPolygon(cpoly, clPnt1) && isPointInPolygon(cpoly, clPnt2));
	}
	
	
	
	private boolean isContourLineIntersectsContour(ContourLine cl, ConcreteContour c){
		//if cl resides in contour then it doesn't intersect and => false is returned
		Polygon cpoly = c.getPolygon();
		Line2D.Double clLine = cl.getLine();
		Point2D.Double clPnt1 = new Point2D.Double (clLine.x1, clLine.y1);
		Point2D.Double clPnt2 = new Point2D.Double (clLine.x2, clLine.y2);
		return ( (isPointInPolygon(cpoly, clPnt1) && !isPointInPolygon(cpoly, clPnt2)) || 
				!(isPointInPolygon(cpoly, clPnt1) && isPointInPolygon(cpoly, clPnt2)) ) ;
	}
	
	
	
	private ArrayList<Zone> getZonesRelatedToContour (ConcreteContour rcc, ArrayList<Zone> zones){
		ArrayList<ConcreteContour> rccs = new ArrayList<ConcreteContour>();
		rccs.add(rcc);
		return (getZonesRelatedToContours(rccs, zones));
	}	
	
	private ArrayList<Zone> getZonesRelatedToContours (ArrayList<ConcreteContour> rccs, ArrayList<Zone> zones){
		ArrayList<Zone> zonesRelatedToContours = new ArrayList<Zone>(); 
		
		for (Zone z : zones){
			String absDesc = z.getAbstractDescription();
			
			boolean zoneRelatedToContour = false;
			for (ConcreteContour rcc : rccs){
				if (absDesc.contains(rcc.getAbstractContour())){
					zoneRelatedToContour = true;
					break;
				}
			}
			
			if (zoneRelatedToContour)
				zonesRelatedToContours.add(z);
		}
		
		return zonesRelatedToContours;
	}
	
	
	
	private ArrayList<Zone> removeZonesInContour (ConcreteContour cc, ArrayList<Zone> zones){
		ArrayList<Zone> zonesNotInContour = new ArrayList<Zone>();
		for (Zone z : zones){
			if (! isZoneInContour(cc, z)){
				zonesNotInContour.add(z);
			}
		}
		return zonesNotInContour;
	}
	
	
	
	private ArrayList<Zone> removeZonesNotInAnyContours (ArrayList<ConcreteContour> ccs, ArrayList<Zone> zones){
		ArrayList<Zone> zonesInAnyContours = new ArrayList<Zone>();

		for (Zone z : zones){
			if (isZoneInAnyOfTheseContours(ccs, z)){
				zonesInAnyContours.add(z);
			}
		}
		return zonesInAnyContours;
	}
	
	
	
	private ArrayList<ConcreteContour> getOtherContoursOfZone (ConcreteContour cc, Zone z, ArrayList<ConcreteContour> allContoursInDiagram){
		ArrayList<Zone> zs = new ArrayList<Zone>();
		zs.add(z);
		
		return getOtherContoursOfZones(cc, zs,allContoursInDiagram);
	}	
	
	private ArrayList<ConcreteContour> getOtherContoursOfZones (ConcreteContour cc, ArrayList<Zone> zones, ArrayList<ConcreteContour> allContoursInDiagram){
		ArrayList<ConcreteContour> ccs = new ArrayList<ConcreteContour>();
		ccs.add(cc);
		
		return getOtherContoursOfZones(ccs, zones, allContoursInDiagram);
	}
	
	private ArrayList<ConcreteContour> getOtherContoursOfZones (ArrayList<ConcreteContour> knownContoursOfZones, ArrayList<Zone> zones, ArrayList<ConcreteContour> allContoursInDiagram){
		
		if (zones == null){return null;}
		
		ArrayList<ConcreteContour> otherContoursOfZones = new ArrayList<ConcreteContour>();
		
				
		ArrayList<String> ccsAbsDesc = new ArrayList<String>();
		for (ConcreteContour cc : knownContoursOfZones){
			ccsAbsDesc.add(cc.getAbstractContour());
		}
		
		
		for (Zone z : zones){
			String zAbsDesc = z.getAbstractDescription();
			
			for (String ccAbsDesc : ccsAbsDesc){
				if (zAbsDesc.contains(ccAbsDesc))
					zAbsDesc = zAbsDesc.replace(ccAbsDesc, "");
				if (zAbsDesc.length() == 0)
					break;
			}
			
			if (zAbsDesc.length() > 0){
				for(int i=0; i < zAbsDesc.length(); i++){
					char ocAbsDesc = zAbsDesc.charAt(i);
					ConcreteContour oc = getConcreteContourFromAbstractDescription(Character.toString(ocAbsDesc), allContoursInDiagram);
				
					if (!otherContoursOfZones.contains(oc))
						otherContoursOfZones.add(oc);
				}
			}
		}
	
		return otherContoursOfZones;
	}
	
	
	
	private ConcreteContour getConcreteContourFromAbstractDescription (String contourAbstractDescription, ArrayList<ConcreteContour> contours){
		for (ConcreteContour cc : contours){
			if (cc.getAbstractContour().equals(contourAbstractDescription))
				return cc;
		}
		return null;
	}
	
	
	
	private boolean isZoneInContour (ConcreteContour cc, Zone z){
		return (z.getAbstractDescription().contains(cc.getAbstractContour()));
	}
	
	
	
	private boolean isZoneInAnyOfTheseContours (ArrayList<ConcreteContour> ccs, Zone z){
		for (ConcreteContour cc : ccs){
			if (isZoneInContour(cc,z))
				return true;
		}
		return false;
	}
	
	
	
	private ArrayList<Zone> getNeighbouringZonesOfIntersectingContours(ConcreteContour cc, ArrayList<Zone> zones, ArrayList<ConcreteContour> ccs){
		
		ArrayList<ConcreteContour> intersectingContours = getIntersectingContours(cc, ccs);	
		ArrayList<Zone> zonesRelatedToICs = getZonesRelatedToContours(intersectingContours, zones);
		ArrayList<Zone> zonesRelatedToICsNotCC = removeZonesInContour(cc, zonesRelatedToICs);
		
		ArrayList<ConcreteContour> otherContoursOfZones = getOtherContoursOfZones(intersectingContours, zonesRelatedToICsNotCC, ccs);
		
		ArrayList<Zone> neighbouringZonesOfICs = new ArrayList<Zone>();
		
		for (Zone zric : zonesRelatedToICsNotCC){
			if (!isZoneInAnyOfTheseContours(otherContoursOfZones, zric)){
				if (!neighbouringZonesOfICs.contains(zric))
					neighbouringZonesOfICs.add(zric);
			}		
		}
		
		return neighbouringZonesOfICs;
		
	}
	
	
	
	private ArrayList<ConcreteContour> getContoursIntersectingWithIntersectingContours (ConcreteContour contour, ArrayList<Zone> zones, ArrayList<ConcreteContour> allContoursInDiagram){
		ArrayList<ConcreteContour> intersectingContours = getIntersectingContours(contour, allContoursInDiagram);	
		ArrayList<Zone> zonesRelatedToICs = getZonesRelatedToContours(intersectingContours, zones);
		ArrayList<Zone> zonesRelatedToICsNotCC = removeZonesInContour(contour, zonesRelatedToICs);
		
		return (getOtherContoursOfZones(intersectingContours, zonesRelatedToICsNotCC, allContoursInDiagram));
	}
	
	
	
	private ArrayList<Point2D.Double> getLineIntersectionPointsWithContour(ConcreteContour cc, MyLine line){
		ArrayList<Point2D.Double> lineIntersectionPointsWithContour = new ArrayList<Point2D.Double>();
		ArrayList<ContourLine> contourLines = cc.getContourLines();
		
		for (ContourLine cl : contourLines){
			Line2D.Double clLine = cl.getLine();
			
			Point2D.Double intPnt = line.getLineIntersectionPointWithLineSeg(new MyLine(clLine.x1, clLine.y1, clLine.x2, clLine.y2));
			if (intPnt != null){ //if contourline segment intersects with line
				lineIntersectionPointsWithContour.add(intPnt);
			}
		}
		
		return lineIntersectionPointsWithContour;
	}	
	
	
	
	private ArrayList<Point2D.Double> getLineIntersectionPointsWithZone (Line2D.Double line, Zone z){
		
		ArrayList<Point2D.Double> lineIntersectionPointsWithZone = new ArrayList<Point2D.Double>();
		ArrayList<Polygon> zpolys = z.getPolygons();
		
		for (Polygon zpoly : zpolys){
			ArrayList<Line2D.Double> zpolylines = getPolygonLines(zpoly);
			for (Line2D.Double zpl : zpolylines){
				Point2D.Double pnt = new Point2D.Double (line.x1, line.y1);
				if (isPointInPolygon(zpoly, pnt)){
					if (!lineIntersectionPointsWithZone.contains(pnt)){
						lineIntersectionPointsWithZone.add(pnt);}
				}
				pnt = new Point2D.Double (line.x2, line.y2);
				if (isPointInPolygon(zpoly, pnt)){
					if (!lineIntersectionPointsWithZone.contains(pnt)){
						lineIntersectionPointsWithZone.add(pnt);}
				}
				if (line.intersectsLine(zpl)){
					pnt = getLineIntersectionPoint(line, zpl);
					if (!lineIntersectionPointsWithZone.contains(pnt)){
						lineIntersectionPointsWithZone.add(pnt);}
				}
			}
		}
		
		return lineIntersectionPointsWithZone;
	}	
	
	
	
	private ArrayList<Point2D.Double> getContourIntersectionPointsWithZone (ConcreteContour cc, Zone z){
		
		ArrayList<Point2D.Double> contourIntersectionPointsWithZone = new ArrayList<Point2D.Double>();
		ArrayList<ContourLine> contourLines = cc.getContourLines();
		
		for (ContourLine cl : contourLines){
			Line2D.Double clLine = cl.getLine();
			ArrayList<Point2D.Double> lineIntPntsWithZone = getLineIntersectionPointsWithZone(clLine, z);
			for (Point2D.Double ip : lineIntPntsWithZone){
				if ((!contourIntersectionPointsWithZone.contains(ip)) && (ip!=null)){
					contourIntersectionPointsWithZone.add(ip);}
			}
		}
		
		return contourIntersectionPointsWithZone;
	}	
	
	
	
	private ArrayList<Point2D.Double> getContoursIntersectionPointsWithZone (ArrayList<ConcreteContour> ccs, Zone z){
		
		ArrayList<Point2D.Double> contoursIntersectionPointsWithZone = new ArrayList<Point2D.Double>();
				
		for (ConcreteContour cc : ccs){
			ArrayList<Point2D.Double> contourIntersectionPointsWithZone = getContourIntersectionPointsWithZone(cc,z);
			for (Point2D.Double ip : contourIntersectionPointsWithZone){
				if ((!contoursIntersectionPointsWithZone.contains(ip)) && (ip!=null)){
					contoursIntersectionPointsWithZone.add(ip);}
			}
		}
		
		return contoursIntersectionPointsWithZone;
	}	
	
	
	
	private ArrayList<Line2D.Double> getLinesIntersectingContour(ConcreteContour cc, Line2D.Double line){
		ArrayList<Line2D.Double> linesIntersectingContour = new ArrayList<Line2D.Double>();
		ArrayList<ContourLine> contourLines = cc.getContourLines();
		
		for (ContourLine cl : contourLines){
			Line2D.Double clLine = cl.getLine();
			if (clLine.intersectsLine(line)){
				linesIntersectingContour.add(line);
			}
		}
		
		return linesIntersectingContour;
	}
		
	
	
	private double[] getAvgZoneAreasForNSets(ArrayList<ConcreteContour> concreteContours, ArrayList<Zone> currentZones){
		int noOfContours = concreteContours.size();
		double[] avgZoneAreasForNSets = new double[noOfContours]; 
		//set the size of the array to the max N sets zone you can have

		double[] totalZoneAreasForNSets = new double[noOfContours];
		ArrayList<String>[] countZonesWithNSets = new ArrayList[noOfContours];
		// initialize array
		for (int j = 0; j < noOfContours; j++){
			totalZoneAreasForNSets[j] = 0.0;
			countZonesWithNSets[j] = new ArrayList<String>();					
		}
		
		// calculate avg zone area for n sets 
		for (Zone cz : currentZones){
			int zoneAbsDescLen = cz.getAbstractDescription().length();
			totalZoneAreasForNSets[zoneAbsDescLen-1] += cz.computeTotalArea();
			if (!countZonesWithNSets[zoneAbsDescLen-1].contains(cz.getAbstractDescription())){
				countZonesWithNSets[zoneAbsDescLen-1].add(cz.getAbstractDescription());
			}
		}
		for (int j = 0; j < noOfContours; j++){
			avgZoneAreasForNSets[j] = totalZoneAreasForNSets[j] / countZonesWithNSets[j].size();
		}
		return avgZoneAreasForNSets;
	}
	
	
	
	
	// Find whether this contour is contained in any other contours(that is there are contours which hold this entire contour and which this contour is a subset of)
	private boolean isContourContainedInOtherContours (ConcreteContour contour, HashMap<ConcreteContour, ArrayList<ConcreteContour>> contoursContainingContours){
		return (getContoursContainingThisContour(contour,contoursContainingContours).size()>0);
	}
	
	
	private boolean doesContourContainOtherContour (ConcreteContour c1, ConcreteContour c2){
		return (ConcreteContour.polygonContainment(c1.getPolygon(), c2.getPolygon()));
	}
	
	
	
	// Find all the contours that contain this contour (that is all the contours which hold this contour and which this contour is a subset of)
	private ArrayList<ConcreteContour> getContoursContainingThisContour (ConcreteContour contour, HashMap<ConcreteContour, ArrayList<ConcreteContour>> contoursContainingContours){
		
		Iterator itr = contoursContainingContours.entrySet().iterator();
		
		ArrayList<ConcreteContour> contoursContainingThisContour = new ArrayList<ConcreteContour>();
		
		while (itr.hasNext()){
			Entry<ConcreteContour, ArrayList<ConcreteContour>> contourListPair = (Entry<ConcreteContour, ArrayList<ConcreteContour>>)itr.next();
			if (contourListPair.getValue().contains(contour)){
				contoursContainingThisContour.add(contourListPair.getKey());
			}
		}
		
		return contoursContainingThisContour;
	}
	
	
	
	// Find all those contours that contain other contours
	private HashMap<ConcreteContour, ArrayList<ConcreteContour>> getContoursContainingContours (ArrayList<ConcreteContour> ccs){
		
		HashMap<ConcreteContour, ArrayList <ConcreteContour>> contoursContainingContours = new HashMap<ConcreteContour, ArrayList <ConcreteContour>>();
		
		for (ConcreteContour c1 : ccs){
			for (ConcreteContour c2 : ccs){
				if (c1==c2){continue;}
				
				if (ConcreteContour.polygonContainment(c1.getPolygon(), c2.getPolygon())){
					if (contoursContainingContours.containsKey(c1)){
						(contoursContainingContours.get(c1)).add(c2);
					} else {
						ArrayList<ConcreteContour> contoursContained = new ArrayList<ConcreteContour>();
						contoursContained.add(c2);
						contoursContainingContours.put(c1, contoursContained);
					}
				}
			}
		}
		
		return contoursContainingContours;
	}
	
	
	
	private ArrayList<Zone> getZonesContainedInZone (Zone zone, ArrayList<Zone> zones){
		ArrayList<Zone> zonesInZone = new ArrayList<Zone>();
		
		for (Zone cz : zones){
			if (cz.getPolygons().size()>0){
				Polygon czpoly0 = cz.getPolygons().get(0);
				for (Polygon zonepoly : zone.getPolygons()){
					if (ConcreteContour.polygonContainment(zonepoly,czpoly0)){
						zonesInZone.add(cz);
					}
				}
			}
		}
		
		return zonesInZone;
	}
	
	
	
	
	
	
	// Protected Methods
		
	private Point2D.Double getMidPnt(Point2D.Double pnt1, Point2D.Double pnt2){
		return new Point2D.Double(((pnt1.x + pnt2.x)/2), ((pnt1.y + pnt2.y)/2));
	}
	
	
	private ArrayList<Point2D.Double> getEquidistantPntsOnLine(Point2D.Double pnt1, Point2D.Double pnt2, int npnts){
		ArrayList<Point2D.Double> equidistantPntsOnLine = new ArrayList<Point2D.Double>();
		for (int i =0; i<npnts; i++){
			equidistantPntsOnLine.add(new Point2D.Double( ((pnt1.x + pnt2.x)*(i/npnts)), ((pnt1.y + pnt2.y)*(i/npnts)) ));
		}
		return equidistantPntsOnLine;
	}
	
	
	private double computeContourArea(ConcreteContour cc){
		return (Utilities.computePolygonArea(cc.getPolygon()));
	}	
	
	
	private Point2D.Double computeContourCentroid (ConcreteContour cc){
		return (convertToPoint2D_Double(Utilities.findMiddlePointInsidePolygon(cc.getPolygon())));
	}
	
	
	// Identify the second point which should be used to calculate the zone middle repulsion force when a point is unintentionally is in a zone 
	protected Point2D.Double getZoneMiddleRepulsion_PointInZone_Pnt2(Point2D.Double pnt, int pntIndexOnContour, int pntContourIndex, Point2D.Double zoneMiddle, HashMap<String, ArrayList<String>>origZonesForPnts, ArrayList<String> zonesnames, ArrayList<ArrayList<Polygon>> zspolys){
		
		Polygon zoneContainingPoint = findClosestZoneContainingPoint(pnt, pntIndexOnContour, pntContourIndex, origZonesForPnts, zonesnames, zspolys);
	    	    
		if (zoneContainingPoint == null){
	    	return zoneMiddle;
	    } else {
	    	Point middleOfZoneContainingPoint = Utilities.findMiddlePointInsidePolygon(zoneContainingPoint);
	    	return (new Point2D.Double(new Double(middleOfZoneContainingPoint.x), new Double(middleOfZoneContainingPoint.y)));
	    }
	}
	
	
	
	//  Force Type
	protected enum ForceType {a, a_cc,  ad, ad2, cm_oc_a, cm_noc_a, zm_iz_a, zm_niz_a, r, r_cc, rd, cm_oc_r, cm_noc_r, zm_iz_r, zm_niz_r};
	
	// Add node's Attraction force to the current node's total force
	protected Point2D.Double addxyAttraction (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2, Double contour1RectAreaValue){
		return (addxyForce(ForceType.a, xyForce, pnt1, pnt2, contour1RectAreaValue));
	}
	protected Point2D.Double addxyAttractionContourCentroid (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.a_cc, xyForce, pnt1, pnt2, null));
	}
	// Add node's Attraction force from internal middle of contour
	protected Point2D.Double addxyContourMiddleAttraction_PointOnContour (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.cm_oc_a, xyForce, pnt1, pnt2, null));
	}
	// Add node's Attraction force from internal middle of contour
	protected Point2D.Double addxyContourMiddleAttraction_PointNotOnContour (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.cm_noc_a, xyForce, pnt1, pnt2, null));
	}
	// Add node's Attraction force from internal middle of contour
	protected Point2D.Double addxyZoneMiddleAttraction_PointInZone (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.zm_iz_a, xyForce, pnt1, pnt2, null));
	}
	// Add node's Attraction force from internal middle of contour
	protected Point2D.Double addxyZoneMiddleAttraction_PointNotInZone (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.zm_niz_a, xyForce, pnt1, pnt2, null));
	}
	
	// Add node's Repulsion force to the current node's total force
	protected Point2D.Double addxyRepulsion (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2, Double contour1RectAreaValue){
		return (addxyForce(ForceType.r, xyForce, pnt1, pnt2, contour1RectAreaValue));
	}
	protected Point2D.Double addxyRepulsionContourCentroid (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.r_cc, xyForce, pnt1, pnt2, null));
	}
	// Add node's Repulsion (between points on different contours) force to the current node's total force
	protected Point2D.Double addxyRepulsionPntsOnDiffContours (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2, Double contour1RectAreaValue){
		return (addxyForce(ForceType.rd, xyForce, pnt1, pnt2, contour1RectAreaValue));
	}
	protected Point2D.Double addxyAttractionPntsOnDiffContours (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2, Double contour1RectAreaValue){
		return (addxyForce(ForceType.ad, xyForce, pnt1, pnt2, contour1RectAreaValue));
	}
	protected Point2D.Double addxyAttractionPntsOnDiffContours2 (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2, Double contour1RectAreaValue){
		return (addxyForce(ForceType.ad2, xyForce, pnt1, pnt2, contour1RectAreaValue));
	}
	// Add node's Repulsion force to the current node's total force
	protected Point2D.Double addxyContourMiddleRepulsion_PointOnContour (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.cm_oc_r, xyForce, pnt1, pnt2, null));
	}
	// Add node's Repulsion force to the current node's total force
	protected Point2D.Double addxyContourMiddleRepulsion_PointNotOnContour (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.cm_noc_r, xyForce, pnt1, pnt2, null));
	}
	// Add node's Repulsion force to the current node's total force
	protected Point2D.Double addxyZoneMiddleRepulsion_PointInZone  (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.zm_iz_r, xyForce, pnt1, pnt2, null));
	}
	// Add node's Repulsion force to the current node's total force
	protected Point2D.Double addxyZoneMiddleRepulsion_PointNotInZone  (Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2){
		return (addxyForce(ForceType.zm_niz_r, xyForce, pnt1, pnt2, null));
	}	
	
	
	// Add node's Attraction (opt ='a') / Repulsion (opt ='r') force to the current node's total force
	protected Point2D.Double addxyForce(ForceType ft, Point2D.Double xyForce, Point2D.Double pnt1, Point2D.Double pnt2, Double contour1RectAreaValue){
	
		Point2D.Double xyDistance = getxyAbsDistances (pnt1, pnt2);
		double distance = getAbsHypotenuse(xyDistance);
		
		double totalForce = 0;
		Point2D.Double xyForceDirection = new Point2D.Double (1,1);
		switch (ft){
			case a : totalForce = totalAttraction(distance, contour1RectAreaValue); 
					 xyForceDirection = getForceDirection (ForceType.a, pnt1, pnt2);
					 break;
			case a_cc : totalForce = totalAttractionContourCentroid(distance); 
			 			xyForceDirection = getForceDirection (ForceType.a_cc, pnt1, pnt2);
			 			break;
			case ad :  totalForce = totalAttractionPntsOnDiffContours (distance, contour1RectAreaValue); 
		          		xyForceDirection = getForceDirection (ForceType.ad, pnt1, pnt2);
		          		break;
			case ad2 : totalForce = totalAttractionPntsOnDiffContours2 (distance, contour1RectAreaValue); 
          				xyForceDirection = getForceDirection (ForceType.ad, pnt1, pnt2);
          				break;
			case cm_oc_a :  totalForce = totalContourMiddleAttraction_PointOnContour(distance); 
			   		    	xyForceDirection = getForceDirection (ForceType.cm_oc_a, pnt1, pnt2);
			   		    	break;
			case cm_noc_a : totalForce = totalContourMiddleAttraction_PointNotOnContour(distance); 
   		    				xyForceDirection = getForceDirection (ForceType.cm_noc_a, pnt1, pnt2);
   		    				break;
			case zm_iz_a :  totalForce = totalZoneMiddleAttraction_PointInZone(distance); 
   							xyForceDirection = getForceDirection (ForceType.zm_iz_a, pnt1, pnt2);
   							break;				            
			case zm_niz_a : totalForce = totalZoneMiddleAttraction_PointNotInZone(distance); 
   		    				xyForceDirection = getForceDirection (ForceType.zm_niz_a, pnt1, pnt2);
   		    				break;		
   		    				
			case r : totalForce = totalRepulsion(distance, contour1RectAreaValue); 
					 xyForceDirection = getForceDirection (ForceType.r, pnt1, pnt2);
					 break;
			case r_cc : totalForce = totalRepulsionContourCentroid (distance); 
	 		            xyForceDirection = getForceDirection (ForceType.r_cc, pnt1, pnt2);
	 		            break;
			case rd : totalForce = totalRepulsionPntsOnDiffContours (distance, contour1RectAreaValue); 
			 		  xyForceDirection = getForceDirection (ForceType.rd, pnt1, pnt2);
			 		  break;
			case cm_oc_r :  totalForce = totalContourMiddleRepulsion_PointOnContour(distance); 
   		    		    	xyForceDirection = getForceDirection (ForceType.cm_oc_r, pnt1, pnt2);
   		    		        break;
			case cm_noc_r : totalForce = totalContourMiddleRepulsion_PointNotOnContour(distance); 
   		    				xyForceDirection = getForceDirection (ForceType.cm_noc_r, pnt1, pnt2);
   		    				break;
			case zm_niz_r : totalForce = totalZoneMiddleRepulsion_PointInZone(distance); 
   							xyForceDirection = getForceDirection (ForceType.zm_iz_r, pnt1, pnt2);
   							break;   
			case zm_iz_r :  totalForce = totalZoneMiddleRepulsion_PointNotInZone (distance);
						    xyForceDirection = getForceDirection (ForceType.zm_niz_r, pnt1, pnt2);
							break;   		    		    
		}
		
		
		Point2D.Double new_xyForce = new Point2D.Double();
	
		new_xyForce.x = xyForceDirection.x * (xyDistance.x/(xyDistance.x+xyDistance.y)) * totalForce;
		new_xyForce.y = xyForceDirection.y * (xyDistance.y/(xyDistance.x+xyDistance.y)) * totalForce;
	
		
		// Check for Infinite or NaN
		
		if (Double.isInfinite(new_xyForce.x)){
			new_xyForce.x = xyForceDirection.x * maxAbsForce;
		} else if (Double.isNaN(new_xyForce.x)){
			new_xyForce.x = 0;
		}
		
		if (Double.isInfinite(new_xyForce.y)){
			new_xyForce.y = xyForceDirection.y * maxAbsForce;
		} else if (Double.isNaN(new_xyForce.y)){
			new_xyForce.y = 0;
		}
		
		
		// Update the force for point 
		
		xyForce.x += new_xyForce.x;
		xyForce.y += new_xyForce.y;
		
		return xyForce;
	}
	
	
	
	// Return force direction that -1 (for -ve -> move left/up) or +1 (for +ve -> move right/down)
	protected Point2D.Double getForceDirection (ForceType ft, Point2D.Double p1, Point2D.Double p2){
		
		Point2D.Double xyForceDirection = new Point2D.Double (1,1);
		
		Point2D.Double pnt1 = new Point2D.Double();
		Point2D.Double pnt2 = new Point2D.Double();
		
		switch (ft){
			case a:
			case a_cc:
			case ad:
			case cm_oc_a:
			case cm_noc_a:
			case zm_iz_a:
			case zm_niz_a:
			case zm_iz_r:   pnt1 = p2; 
					  	    pnt2 = p1;
					        break;
			case r:
			case r_cc:
			case rd:
			case cm_oc_r:
			case cm_noc_r:
			case zm_niz_r:  pnt1 = p1; 
			  		        pnt2 = p2;
			  		        break;
		}
		
		if (pnt1.x < pnt2.x) 
			xyForceDirection.x = -1;
	
		if (pnt1.y < pnt2.y)
			xyForceDirection.y = -1;
		
		return xyForceDirection;
	}
	
	
	
	// Update the position of the nodes after the forces are considered 
	protected Point2D.Double getNewPntPos (Point2D.Double xyForce, Point2D.Double pnt){
		
		// Ensure that the force to be applied is between limits 
		
		if (xyForce.x > maxAbsForce){
			xyForce.x = maxAbsForce;
		} else if (xyForce.x < (-maxAbsForce)){
			xyForce.x = - maxAbsForce;
		}
		
		if (xyForce.y > maxAbsForce){
			xyForce.y = maxAbsForce;
		} else if (xyForce.y < (-maxAbsForce)){
			xyForce.y = - maxAbsForce;
		}
			
		// Get the new position for the point 
		
		Point2D.Double newPos = new Point2D.Double();
		
		newPos.x = Math.rint(m * xyForce.x) + pnt.x; 
		newPos.y = Math.rint(m * xyForce.y) + pnt.y;
		
		return (newPos);
	}
	

	
	// Find and return the first zone polygons which contains the given point and which is not part of the null (outer) zone
	protected Polygon findFirstZoneContainingPoint (int pntIndexOnContour, int pntContourIndex, HashMap<String, ArrayList<String>>origZonesForPnts, ArrayList<String> zonesNames, ArrayList<ArrayList<Polygon>> zspolys){

		String pntID = Integer.toString(pntContourIndex) + "|" +  Integer.toString(pntIndexOnContour);
		
		ArrayList<String> pntOrigInZone = ((ArrayList<String>)origZonesForPnts.get(pntID));
		int i = 0;
		int zoneIndex = 0;
		String reqZoneName = "";
		do{
			reqZoneName = pntOrigInZone.get(i);
			zoneIndex = zonesNames.indexOf(reqZoneName);
			i++;
		} while ((reqZoneName == "") && (i < pntOrigInZone.size()));
		
		try{
			if ((i <= pntOrigInZone.size()) && (reqZoneName != "")){
				return (zspolys.get(zoneIndex).get(0));
			} else {
				return null;
			}
		}catch (IndexOutOfBoundsException e){
			return null;
		}
	}
	
	
	
	// Find and return the closest zone centroid which contains the given point
	protected Point2D.Double findClosestZoneCentroidContainingPoint (Point2D.Double pnt, int pntIndexOnContour, int pntContourIndex, HashMap<String, ArrayList<Zone>>zonesForPnts){

		String pntID = Integer.toString(pntContourIndex) + "|" + Integer.toString(pntIndexOnContour);
		ArrayList<Zone> zonesForPnt = ((ArrayList<Zone>)zonesForPnts.get(pntID));

		Point2D.Double closestCentroid = null;
		double closestCentroidDistance = -1;
		for (Zone z : zonesForPnt){
			ArrayList<Point2D.Double> zoneCentroids = z.computeCentroids();
			for (Point2D.Double c : zoneCentroids){
				double pntToCentroidDistance = getAbsDistance(pnt, c);
				if ((closestCentroidDistance < 0) || (closestCentroidDistance > pntToCentroidDistance)){
					closestCentroidDistance = pntToCentroidDistance;
					closestCentroid = c;
				}
			}
		}
		
		return closestCentroid;
	}
	
	
	
	// Find and return the closest zone polygons which contains the given point and which is not part of the null (outer) zone
	protected Polygon findClosestZoneContainingPoint (Point2D.Double pnt, int pntIndexOnContour, int pntContourIndex, HashMap<String, ArrayList<String>>origZonesForPnts, ArrayList<String> zonesNames, ArrayList<ArrayList<Polygon>> zspolys){

		String pntID = Integer.toString(pntContourIndex) + "|" + Integer.toString(pntIndexOnContour);
		
		ArrayList<String> pntOrigInZone = ((ArrayList<String>)origZonesForPnts.get(pntID));

		Polygon closestPoly = null;
		double closestPolyDistance = -1;
		try{
			for (String zoneName : pntOrigInZone){
				if (zoneName == ""){continue;}
			
				int zoneIndex = zonesNames.indexOf(zoneName);
				ArrayList<Polygon> zonePolys = zspolys.get(zoneIndex);
			
				for (Polygon zonePoly : zonePolys){
					Point zoneInternalMiddle = Utilities.findMiddlePointInsidePolygon(zonePoly);
					Point2D.Double zoneInternalMiddle2D = new Point2D.Double (new Double(zoneInternalMiddle.x), new Double(zoneInternalMiddle.y)); 
					Double absDistance = getAbsDistance(pnt, zoneInternalMiddle2D);
					if ((closestPolyDistance < 0)|| (closestPolyDistance > absDistance)){
						closestPolyDistance = absDistance;
						closestPoly = zonePoly;
					}
				}
			}
			return closestPoly;
		}catch (IndexOutOfBoundsException e){
			return null;
		}
	}
	
	
	
	// Return the length of a line 
	protected Double getLineLength (Line2D.Double line){
		return getAbsHypotenuse(getxyAbsDistances(new Point2D.Double(line.x1, line.y1), new Point2D.Double(line.x2, line.y2)));
	}
	
	
	
	// Calculate & return the absolute distance between two points
	protected Double getAbsDistance (Point2D.Double pnt1, Point2D.Double pnt2){
		return getAbsHypotenuse(getxyAbsDistances(pnt1, pnt2));
	}
	
	
	
	// Calculate & return the absolute distance between the nodes on the x and y axis
	protected Point2D.Double getxyAbsDistances (Point2D.Double pnt1, Point2D.Double pnt2){
		Point2D.Double xyAbsDistances = getxyDistances(pnt1, pnt2);
		xyAbsDistances.x = Math.abs(xyAbsDistances.x);
		xyAbsDistances.y = Math.abs(xyAbsDistances.y);
		return xyAbsDistances;
	}
	
	
	
	// Calculate & return the distance between the nodes on the x and y axis
	protected Point2D.Double getxyDistances (Point2D.Double pnt1, Point2D.Double pnt2){
		Point2D.Double xyDistances = new Point2D.Double();
		
		xyDistances.x = pnt1.x-pnt2.x;
		xyDistances.y = pnt1.y-pnt2.y;
		
		return xyDistances;
	}
	
	
	
	// Calculate & return the absolute distance between the nodes 
	//     considering it as a hypotenuse of a triangle
	protected Double getAbsHypotenuse (Point2D.Double xyDistance){
		return (Math.sqrt(Math.pow(xyDistance.x, 2)+ Math.pow(xyDistance.y, 2)));
	}
	
	
	
	// Methods which might be useful especially for debugging
	//     most of these can potentially be set to public static 
	private void printContoursCoordinates (ArrayList<ConcreteContour> ccs){
		for (ConcreteContour c : ccs){
			Polygon cPoly = c.getPolygon();
			for (int p = 0; p < cPoly.npoints; p++){
				System.out.println (c.getAbstractContour() + " -> (" + cPoly.xpoints[p] +","+cPoly.ypoints[p]+")");
			}
		}
	}
	
	
	// print the properties of the polygons making up the curves (also referred to here as contours) of an Euler diagram
	private void printZonesForContoursPoints (ArrayList<ConcreteContour> ccs, HashMap<String, ArrayList<Zone>> zonesForPnts){
		
		int ci =0;
		for (ConcreteContour c : ccs){
			Polygon cPoly= c.getPolygon();
			
			for (int p = 0; p < cPoly.npoints; p++){
				
				String nextPntKey = Integer.toString(ci)+"|"+Integer.toString(p);
				ArrayList<Zone> nextPntZones = (ArrayList<Zone>)zonesForPnts.get(nextPntKey);
				System.out.println ("Point " + nextPntKey + ": ");
				
				if (nextPntZones !=null){
					for (Zone z : nextPntZones){
						System.out.println ("      - " + z.getAbstractDescription());
					}
				}
			}
			ci++;
		}
	}
	
	
	
	// print the properties of the polygons making up the zones
	private void printZonesPolygonsOnly (ArrayList<ArrayList<Polygon>> zsPolys){
		int zc = 0;
		int zpc = 0;
		
		ArrayList<ConcreteContour> origConcreteContours = (ArrayList<ConcreteContour>) concreteContours.clone();
		ArrayList<ConcreteContour> tempConcreteContours = new ArrayList<ConcreteContour>();
		
		for (ArrayList<Polygon> zPolys : zsPolys){
			for (Polygon zPoly : zPolys){
				tempConcreteContours.add(new ConcreteContour(Integer.toString(zc)+Integer.toString(zpc), zPoly));
				zpc++;
				
				simpleDiagramPanel.simpleConcreteDiagram.setConcreteContours(tempConcreteContours);
				simpleDiagramPanel.updateDisplay();
			}
			zc++;
			
		}
		simpleDiagramPanel.simpleConcreteDiagram.setConcreteContours(tempConcreteContours);
		simpleDiagramPanel.updateDisplay();
		
		// this is just to ensure that the original contours are not lost
		simpleDiagramPanel.simpleConcreteDiagram.setConcreteContours(origConcreteContours);
	}
}




