/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;

import java.applet.Applet;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Area;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JPanel;
import javax.swing.event.MouseInputListener;

import euler.ConcreteContour;
import euler.utilities.DiagramUtility;


/**
 * To instantiate, define and handle the panel in which Euler diagrams are displayed in eulerForce
 * 
 */


public class SimpleDiagramPanel extends JPanel implements MouseInputListener, KeyListener {

	// Default Class Values 
	public static final Color PANELBACKGROUNDCOLOR = Color.white;
	//     to draw contours
	public static final String LABELFONTNAME = "Arial";
	public static final int LABELFONTSTYLE = Font.BOLD;
	public static final int LABELFONTSIZE = 12;
	public static final int CONTOURLABELFONTSIZE = 18;
	public static Color c1 = new Color(200,100,100); 
	public static Color c2 = new Color(0,200,100);
	public static Color c3 = new Color(100,0,200); 
	public static Color c4 = new Color(200,100,0); 
	public static Color contourColors[] = {Color.RED,Color.GREEN,Color.CYAN,Color.GRAY,Color.MAGENTA,Color.BLACK,
		Color.PINK,Color.ORANGE,Color.LIGHT_GRAY,Color.YELLOW,Color.DARK_GRAY,c1,c2,c3,c4};
	
	
	// Object Data Members
	
	protected SimpleConcreteDiagram simpleConcreteDiagram;
	
	//    panel group features
	protected ArrayList<SimpleDiagramDrawer> simpleDiagramDrawerList = new ArrayList<SimpleDiagramDrawer>();
	protected ArrayList<DiagramUtility> diagramUtilityList = new ArrayList<DiagramUtility>();
	
	
	//    panel setup properties
	protected Applet applet;
	protected Frame containerFrame;
	protected Color panelBackgroundColor = PANELBACKGROUNDCOLOR;

	
	//    inherited
	private static final long serialVersionUID = 1L;
	
	//    other 
	protected StopWatch stopWatch = new StopWatch(); 
	protected boolean showStopWatch = false;
	
	protected boolean showDiagramDetails = false;
	protected String abstractDescInitialLayout;
	protected boolean finalLayout = false;
	
	protected int winWidth;
	protected int winHeight;
	
	
	// Constructors
	
	public SimpleDiagramPanel(SimpleConcreteDiagram inSimpleConcreteDiagram, Applet inApplet){
		simpleConcreteDiagram = inSimpleConcreteDiagram;
		
		applet = inApplet;
		containerFrame = null;
		
		setupPanel();
	}
	
	public SimpleDiagramPanel(SimpleConcreteDiagram inSimpleConcreteDiagram, Frame inContainerFrame){
		simpleConcreteDiagram = inSimpleConcreteDiagram;
		
		applet = null;
		containerFrame = inContainerFrame;
		
		setupPanel();
	}
	
	public void setupPanel(){
		setBackground(panelBackgroundColor);
		addMouseListener(this);
		addKeyListener(this);
	}
	
	
	
	// Properties 
	
	public Applet getApplet(){
		return applet;
	}
	
	public Frame getContainerFrame(){
		return containerFrame;
	}
	
	public SimpleConcreteDiagram getSimpleConcreteDiagram(){
		return simpleConcreteDiagram;
	}
	public void setSimpleConcreteDiagram(SimpleConcreteDiagram inSimpleConcreteDiagram){
		simpleConcreteDiagram = inSimpleConcreteDiagram;
	}
	
	public ArrayList<SimpleDiagramDrawer> getSimpleDiagramDrawerList(){
		return simpleDiagramDrawerList;
	}
	public void setSimpleDiagramDrawerList(ArrayList<SimpleDiagramDrawer> inSimpleDiagramDrawerList){
		simpleDiagramDrawerList = inSimpleDiagramDrawerList;
	}
	
	
	public ArrayList<DiagramUtility> getDiagramUtilityList(){
		return diagramUtilityList;
	}
	public void setDiagramUtilityList(ArrayList<DiagramUtility> inDiagramUtilityList){
		diagramUtilityList = inDiagramUtilityList;
	}	
	
	public StopWatch getStopWatch (){
		return stopWatch;
	}
	
	public void setShowStopWatch (boolean inShowStopWatch){
		showStopWatch = inShowStopWatch;
	}
	public boolean getShowStopWatch (){
		return showStopWatch;
	}
	
	public void setShowDiagramDetails (boolean inShowDiagramDetails){
		showDiagramDetails = inShowDiagramDetails;
	}
	public boolean getShowDiagramDetails (){
		return showDiagramDetails;
	}
	
	public void recordAsInitialLayout(){
		abstractDescInitialLayout = ConcreteContour.generateAbstractDiagramFromList(simpleConcreteDiagram.getConcreteContours());
		finalLayout = false;
	}
	public void recordAsFinalLayout(){
		finalLayout = true;
	}

	
	
	public void setWinWidth (int inWinWidth){
		winWidth = inWinWidth;
	}
	public int getWinWidth (){
		return winWidth;
	}
	
	public void setWinHeight (int inWinHeight){
		winHeight = inWinHeight;
	}
	public int getWinHeight(){
		return winHeight;
	}
	
	
	
	// Methods 
	
	/** Add a drawing algorithm to the panel. */
	public void addSimpleDiagramDrawer(SimpleDiagramDrawer sdd) {
		simpleDiagramDrawerList.add(sdd);
		sdd.setSimpleDiagramPanel(this);
	}

	/** Removes a drawing algorithm from the panel. */
	public void removeSimpleDiagramDrawer(SimpleDiagramDrawer sdd) {
		simpleDiagramDrawerList.remove(sdd);
		sdd.setSimpleDiagramPanel(null);
	}
	

	
	/** Add a drawing algorithm to the panel. */
	public void addDiagramUtility(DiagramUtility du) {
		diagramUtilityList.add(du);
	}

	/** Removes a drawing algorithm from the panel. */
	public void removeDiagramUtility(DiagramUtility du) {
		diagramUtilityList.remove(du);
	}
	
	
	/** Is the string in an array of strings? */
	private boolean stringInArray (ArrayList<String> strArray, String requiredStr){
		for (String s: strArray){
			if (s.equals(requiredStr)){ 
				return true;
			}
		}
		return false;
	}
	
	
	
	
	/** Update the display panel */
	public void updateDisplay(){
		paintImmediately(getBounds()); 	
	}
	
	
	/** paint */
	public void paint(Graphics g) {
		paintComponent(g);
	}
	
	
	/** paintComponent */
	public void paintComponent(Graphics g){
		
		Graphics2D g2D = (Graphics2D) g; 	
		super.paintComponent(g2D);

		// Draw/Paint Contours 
		ArrayList<ConcreteContour> concreteContours = simpleConcreteDiagram.getConcreteContours();
		for (ConcreteContour cc : concreteContours){
			paintContour(g2D, cc);
		}	
		
		if (showStopWatch && (concreteContours.size()>0)){
			g2D.setFont(new Font("Arial", Font.BOLD, 20));
			g2D.setColor(Color.black);
			g2D.drawString(String.valueOf(stopWatch.getElapsedTimeString()), winWidth-160, winHeight-80);
		}
		if (showDiagramDetails && (concreteContours.size()>0)){
			
			//upper line
			g2D.setColor(Color.LIGHT_GRAY);
			g2D.drawLine(0, winHeight-125, winWidth-200, winHeight-125); 
			
			//initial diagram
			g2D.setFont(new Font("Arial", Font.BOLD, 14));
			g2D.setColor(Color.darkGray);
			g2D.drawString("Initial Layout:", 15,winHeight-98);
			
			//separator
			g2D.setColor(Color.LIGHT_GRAY);
			int xDescLineSep = (int) winWidth/4 + 20;
			g2D.drawLine(xDescLineSep, winHeight-125, xDescLineSep, winHeight);
			
			//current layout 
			g2D.setFont(new Font("Arial", Font.BOLD, 14));
			g2D.setColor(Color.darkGray);
			String currentLayoutTitle = "Current Layout";
			if (finalLayout){currentLayoutTitle = "Final Layout";}
			g2D.drawString(currentLayoutTitle+":", xDescLineSep+15, winHeight-98);
			if (finalLayout){finalLayout = false;}

			String abstractDescFinalLayout = ConcreteContour.generateAbstractDiagramFromList(simpleConcreteDiagram.getConcreteContours());

			
			//separator
			g2D.setColor(Color.LIGHT_GRAY);
			int xDescValidityLineSep = xDescLineSep * 2;
			g2D.drawLine(xDescValidityLineSep, winHeight-125, xDescValidityLineSep, winHeight);
			
			//validity text 
			g2D.setFont(new Font("Arial", Font.BOLD, 20));
			String validityText="";
			if (abstractDescInitialLayout.equals(abstractDescFinalLayout)){
				g2D.setColor(Color.green);
				validityText = "Valid";
			} else{
				g2D.setColor(Color.red);
				validityText = "Invalid";
			}	
			g2D.drawString(validityText, xDescValidityLineSep+25,winHeight-85);
			int textWidthInPixels = ((int)g2D.getFontMetrics().getStringBounds(validityText, null).getWidth());
			
			paintInvalidAbstractDescription (g2D, 15, winHeight-75, xDescLineSep+15, winHeight-75, xDescValidityLineSep + 25 + textWidthInPixels + 25,winHeight-85);
		}			
	}
	
	

	/** Paint for layouts that are invalid, as they depict the incorrect set of zones with respect to the abstract description of the initial given layout */
	private void paintInvalidAbstractDescription (Graphics g2D, int xStartInitialDesc, int yStartInitialDesc, int xStartCurrDesc, int yStartCurrDesc, int xStartValidity, int yStartValidity){
		String initialDesc = abstractDescInitialLayout.trim() + " ";
		String finalDesc = ConcreteContour.generateAbstractDiagramFromList(simpleConcreteDiagram.getConcreteContours()).trim() + " ";
		
		ArrayList<String> initialZones = new ArrayList<String>();
		ArrayList<String> finalZones = new ArrayList<String>();
		ArrayList<String> newZones = new ArrayList<String>();
		ArrayList<String> missingZones = new ArrayList<String>();
		
			
		String currStr = "";
		for (int i =0; i< initialDesc.length(); i++){
			String currChar = Character.toString(initialDesc.charAt(i));
			if (currChar.equals(" ")){
				initialZones.add(currStr);
				currStr = "";
			} else {
				currStr += currChar; 
			}
		}
		
		currStr = "";
		for (int i =0; i< finalDesc.length(); i++){
			String currChar = Character.toString(finalDesc.charAt(i));
			if (currChar.equals(" ")){
				finalZones.add(currStr);
				currStr = "";
			} else {
				currStr += currChar; 
			}
		}
		
		for (String finalZone : finalZones){
			if (!initialZones.contains(finalZone)) {newZones.add(finalZone);}
		}
		for (String initialZone : initialZones){
			if (!finalZones.contains(initialZone)) {missingZones.add(initialZone);}
		}
		
		
		// drawing 
		// 	  initial description
		g2D.setFont(new Font("Arial", Font.PLAIN, 14));
		int textWidthInPixels = 0; 
		for (String initialZone : initialZones){
			if (stringInArray(missingZones,initialZone)){ 
				g2D.setColor(Color.blue);
			} else {
				g2D.setColor(Color.darkGray);
			}
			g2D.drawString(initialZone, xStartInitialDesc + textWidthInPixels, yStartInitialDesc);
			textWidthInPixels += ((int)g2D.getFontMetrics().getStringBounds(initialZone + " ", null).getWidth());
		}
		
		//    current description
		g2D.setFont(new Font("Arial", Font.PLAIN, 14));
		textWidthInPixels = 0; 
		for (String finalZone : finalZones){
			if (stringInArray(newZones,finalZone)){
				g2D.setColor(Color.magenta);
			} else {
				g2D.setColor(Color.darkGray);
			}
			g2D.drawString(finalZone, xStartCurrDesc + textWidthInPixels, yStartCurrDesc);
			textWidthInPixels += ((int)g2D.getFontMetrics().getStringBounds(finalZone + " ", null).getWidth());
		}
		
		
		//    validity
		
		if (!initialDesc.equals(finalDesc)){
		
		g2D.setFont(new Font("Arial", Font.BOLD, 12));
		g2D.setColor(Color.magenta);
		g2D.drawString("New Zones: ", xStartValidity, yStartValidity-10);
		textWidthInPixels = ((int)g2D.getFontMetrics().getStringBounds("New Zones: ", null).getWidth()); 
		String newZonesStr = " ";
		for (String newZone : newZones){
			newZonesStr += newZone + " ";
		}
		if (newZonesStr.trim().length() == 0){
			newZonesStr = " - ";
		}
		g2D.setFont(new Font("Arial", Font.PLAIN, 12));
		g2D.setColor(Color.darkGray);
		g2D.drawString(newZonesStr, xStartValidity + textWidthInPixels, yStartValidity-10);
		
		g2D.setFont(new Font("Arial", Font.BOLD, 12));
		g2D.setColor(Color.blue);
		g2D.drawString("Missing Zones: ", xStartValidity, yStartValidity+10);
		textWidthInPixels = ((int)g2D.getFontMetrics().getStringBounds("Missing Zones: ", null).getWidth());
		String missingZonesStr = " ";
		for (String missingZone : missingZones){
			missingZonesStr += missingZone + " ";
		}
		if (missingZonesStr.trim().length() == 0){
			missingZonesStr = " - ";
		}
		g2D.setFont(new Font("Arial", Font.PLAIN, 12));
		g2D.setColor(Color.darkGray);
		g2D.drawString(missingZonesStr, xStartValidity + textWidthInPixels, yStartValidity+10);	
		}
	}
	
	
	/** paint a contour */
	private void paintContour (Graphics2D g2D, ConcreteContour cc){
		
		if (cc == null){
			System.out.println("Could not Paint Contour cause concreteContour == null");
			return;
		}
		
		Polygon ccPoly = cc.getPolygon();
		if (ccPoly == null){
			System.out.println("No Contour Polygon to Paint inner Area");
			return;
		}
		
		String label = cc.getAbstractContour();
		
		// Fill in area of polygons
		char contourChar = label.charAt(0);
		Color solidColor = getColorFromChar(contourChar);
		Color transparentColor = new Color(solidColor.getRed()/255,solidColor.getGreen()/255,solidColor.getBlue()/255,0.2f);
		
		g2D.setColor(transparentColor);
		g2D.fill(new Area(ccPoly));
		
		
		g2D.setColor(solidColor);
		BasicStroke basicStroke = getStrokeFromChar(contourChar);
		g2D.setStroke(basicStroke);
				
		g2D.drawPolygon(ccPoly);
		HashMap<String,Polygon> contourPolygonMap = new HashMap<String,Polygon>();
		contourPolygonMap.put(label,ccPoly);
		
		// Show contour label
 	    Font font = new Font(LABELFONTNAME,LABELFONTSTYLE,CONTOURLABELFONTSIZE);
 	    FontRenderContext frc = g2D.getFontRenderContext();
		TextLayout labelLayout = new TextLayout(label, font, frc);
		Point2D.Double labelPoint = findContourLabelPosition(cc);
					
		int labelX = pjr.graph.Util.convertToInteger(labelPoint.x);
		int labelY = pjr.graph.Util.convertToInteger(labelPoint.y);

		Rectangle2D bounds = labelLayout.getBounds();
		bounds.setRect(bounds.getX()+labelX-2, bounds.getY()+labelY-2, bounds.getWidth()+4,bounds.getHeight()+4);

		g2D.setColor(PANELBACKGROUNDCOLOR);
		g2D.fill(bounds);
		g2D.setColor(solidColor);
		labelLayout.draw(g2D,labelX,labelY);
		
		HashMap<String,Color> contourSolidColourMap = new HashMap<String,Color>();
		HashMap<String,Rectangle2D> contourBoundsMap = new HashMap<String,Rectangle2D>();
		HashMap<String,TextLayout> contourLabelLayoutMap = new HashMap<String,TextLayout>();
		HashMap<String,Point2D.Float> contourLabelPointMap = new HashMap<String,Point2D.Float>();
		
		contourSolidColourMap.put(label,solidColor);
		contourBoundsMap.put(label,bounds);
		contourLabelLayoutMap.put(label,labelLayout);
		contourLabelPointMap.put(label,new Point2D.Float(labelX,labelY));	
	}
	
	
	

	/** Returns the color for the contour label */
	public static Color getColorFromChar(char c) {
		int i = (int)c - (int)'a';
		if(i < 0) {i = 0-i;} // capitals give a negative i
		
		int colorInt = i % contourColors.length;
	    Color color = contourColors[colorInt];
		while(i >= contourColors.length) {
			color.brighter();
			i = i - contourColors.length;
		}
		return color;
	}
	

	/** Returns the stroke for the contour label */
	public static BasicStroke getStrokeFromChar(char c) {
	    BasicStroke basicStroke = new BasicStroke(2);
		int i = (int)c - (int)'a';
		if(i < 0) {i = 0-i;} // capitals give a negative i
		
		while(i >= contourColors.length) {
			basicStroke = new BasicStroke(2,BasicStroke.CAP_BUTT,BasicStroke.JOIN_ROUND,0,new float[] {9},0);
			i = i - contourColors.length;
		}
		return basicStroke;
	}
	
	
	/**
	 * Given a polygon, this method finds the point on the polygon closest
	 * to the origin and returns a point on the line connecting to that point.
	 */
	public static Point2D.Double findContourLabelPosition(ConcreteContour cc) {
		
		Polygon p = cc.getPolygon();
		
		if(p == null) {
			return null;
		}
		
		Point2D.Double origin = new Point2D.Double(0,0);
		double points[] = new double[2];
		
		Point2D.Double topLeft = null;
		Point2D.Double previousToTopLeft = null;
		Point2D.Double prev = null;
		double distance = Double.MAX_VALUE;
		
		PathIterator pit = p.getPathIterator(null);
		while(!pit.isDone()) {
			pit.currentSegment(points);
			Point2D.Double nextPoint = new Point2D.Double(points[0],points[1]);
			double nextDistance = pjr.graph.Util.distance(origin,nextPoint);

			if(nextDistance < distance) {
				topLeft = nextPoint;
				previousToTopLeft = prev;
				distance = nextDistance;
			}
			prev = nextPoint;
			pit.next();
			// in case the topLeft is the first point
			if(previousToTopLeft == null) {
				if(pit.isDone()) {
					PathIterator pit2 = p.getPathIterator(null);
					pit2.next(); // at least 2 points in a polygon, start point is the same as the end
					pit2.currentSegment(points);
					previousToTopLeft = new Point2D.Double(points[0],points[1]);
				}
			}

		}
		// split the two points
		Point2D.Double ret = pjr.graph.Util.midPoint(topLeft, previousToTopLeft);
		return ret;
	}
	
	
	
	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
}
