/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;

import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.imageio.ImageIO;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.tools.ant.launch.Launcher;

import euler.DiagramPanel;
import euler.utilities.DiagramUtility;
import euler.utilities.DiagramUtilityRandomWellformedDiagram;


/**
 * To instantiate, define and handle the main window where eulerForce is run
 *  
 * This class starts eulerForce
 * 
 */


public class SimpleDiagramWindow extends JFrame implements ActionListener {

	// Default Class Values 
	public static final String DEFAULT_WIN_TITLE = "eulerForce: Laying Out Euler Diagrams Using a Force-Directed Approach"; 
	
	
	// Data Members
	private SimpleDiagramPanel simpleDiagramPanel;
	
	//    window setup properties
	protected int width;
	protected int height;
	
	//    file properties for saving and loading diagrams
	protected File currentFile = null;
	protected File startDirectory = null;
	protected String currentUserDir="";
	protected File tempDir = new File ("Temp-ForceDirectedLayout");
	protected String tempInitialLayoutName = "InitialLayout";
	protected String tempFinalLayoutName = "FinalLayout";
	
	protected SimpleConcreteDiagram initialSimpleConcreteDiagram;
	protected SimpleConcreteDiagram finalSimpleConcreteDiagram;
	
	
	protected boolean showDesc = false;
	protected boolean showStopwatch = false;
	static boolean showStopWatchDefault = true;
	
	JCheckBoxMenuItem viewShowDescCheckBoxItem;
	JCheckBoxMenuItem viewShowStopwatchCheckBoxItem;
	JMenuItem viewInitialLayoutItem;
	JMenuItem viewFinalLayoutItem;
	
	
	//    inherited
	private static final long serialVersionUID = 1L;

	
	
	
	public static void main(String[] args) {

		//Since layout algo doesn't work properly when run for the first time, initially (before window appears) ...
		//...generate an empty concreteDiagram (new SimpleConcreteDiagram()) 
		SimpleDiagramWindow sdw = new SimpleDiagramWindow (new SimpleConcreteDiagram(), true);
		
		//...and run the layout algo on it 
		ArrayList<SimpleDiagramDrawer> simpleDiagramDrawerList = sdw.getSimpleDiagramPanel().getSimpleDiagramDrawerList();
		sdw.getSimpleDiagramPanel().setShowStopWatch(false);
		sdw.initialSimpleConcreteDiagram = null;
		sdw.finalSimpleConcreteDiagram = null;
		sdw.initialSimpleConcreteDiagram = sdw.simpleDiagramPanel.getSimpleConcreteDiagram().clone();
		for(SimpleDiagramDrawer sdd : simpleDiagramDrawerList) {
			if (sdd.getMenuText().equals(SimpleDiagramDrawerForceModel.DEFAULT_MENUTEXT)) {
				sdd.layout();
			}
		}
		sdw.viewShowDescCheckBoxItem.setSelected(true);
		sdw.viewShowStopwatchCheckBoxItem.setSelected(showStopWatchDefault);
		sdw.setShowDesc();
		sdw.simpleDiagramPanel.getStopWatch().reset();
		sdw.setShowStopwatch();
		sdw.simpleDiagramPanel.updateDisplay();
	}
	
	
	
	
	// Constructors 
	
	public SimpleDiagramWindow(SimpleConcreteDiagram inSimpleConcreteDiagram, boolean visible){
		super(DEFAULT_WIN_TITLE);
		
		BufferedImage icon=null;
		
		try {
			icon = ImageIO.read(getClass().getResourceAsStream("/forcedirected/eulerForce.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		setIconImage(icon);
	
		simpleDiagramPanel = new SimpleDiagramPanel(inSimpleConcreteDiagram, this);
		getContentPane().add(simpleDiagramPanel);
		
		setupFileProperties();
		setupWindow(visible);
		
		simpleDiagramPanel.setWinWidth(width);
		simpleDiagramPanel.setWinHeight(height);
		simpleDiagramPanel.requestFocus();

		repaint();
	}


	private void setupFileProperties(){
		//file properties for saving and loading diagrams
		String startDirectoryName = System.getProperty("user.dir");
		startDirectory = new File(startDirectoryName);
	}
	
	
	private void setupWindow(boolean visible){

		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		// 	setting up Menu Bar
		initUtility();
		initLayout();
		initMenu();
		
		//  setting up Window Size, Location and Visibility
		Dimension frameDim = Toolkit.getDefaultToolkit().getScreenSize();
		
		width = (int)(frameDim.width*.95);
		height = (int)(frameDim.height*.95);
		setSize(width,height);

		int posX = 0;
		int posY = 0;
		setLocation(posX, posY);

		setVisible(visible);
	}
	
	public SimpleDiagramPanel getSimpleDiagramPanel() {return simpleDiagramPanel;}
	
	public void setSimpleDiagramPanel (SimpleDiagramPanel inSimpleDiagramPanel) {simpleDiagramPanel = inSimpleDiagramPanel;}
	
	public void setShowDesc () {
		showDesc = viewShowDescCheckBoxItem.isSelected();
		if (simpleDiagramPanel.getSimpleConcreteDiagram().getConcreteContours().size()>0){
			simpleDiagramPanel.setShowDiagramDetails(showDesc);
			simpleDiagramPanel.updateDisplay();
		}
	}
	
	public void setShowStopwatch () {
		showStopwatch = viewShowStopwatchCheckBoxItem.isSelected();
		if (simpleDiagramPanel.getSimpleConcreteDiagram().getConcreteContours().size()>0){
			simpleDiagramPanel.setShowStopWatch(showStopwatch);
			simpleDiagramPanel.updateDisplay();
		}
	}

	private void initUtility() {
		simpleDiagramPanel.addDiagramUtility(new DiagramUtilityRandomWellformedDiagram(KeyEvent.VK_F2, "with 2 curves",KeyEvent.VK_F2,2,false));
		simpleDiagramPanel.addDiagramUtility(new DiagramUtilityRandomWellformedDiagram(KeyEvent.VK_F3, "with 3 curves",KeyEvent.VK_F3,3,false));
		simpleDiagramPanel.addDiagramUtility(new DiagramUtilityRandomWellformedDiagram(KeyEvent.VK_F4, "with 4 curves",KeyEvent.VK_F4,4,false));
		simpleDiagramPanel.addDiagramUtility(new DiagramUtilityRandomWellformedDiagram(KeyEvent.VK_F5, "with 5 curves",KeyEvent.VK_F5,5,false));
	}
	
	private void initLayout() {
		simpleDiagramPanel.addSimpleDiagramDrawer(new SimpleDiagramDrawerForceModel());
	}
	

	private void initMenu() {
		
		JMenuBar menuBar = new JMenuBar();

		setJMenuBar(menuBar);

		// File Menu
		JMenu fileMenu = new JMenu("File");

		fileMenu.setMnemonic(KeyEvent.VK_L);
		menuBar.add(fileMenu);
	
		JMenuItem fileNewItem = new JMenuItem("New",KeyEvent.VK_N);
		fileNewItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.ALT_MASK));
		fileMenu.add(fileNewItem);
		fileNewItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				fileNew();
			}
		});
		
		JMenuItem fileOpenItem = new JMenuItem("Open",KeyEvent.VK_O);
		fileOpenItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.ALT_MASK));
		fileMenu.add(fileOpenItem);
		fileOpenItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				fileOpen();
			}
		});

		JMenuItem fileSaveAsItem = new JMenuItem("Save",KeyEvent.VK_S);
		fileSaveAsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
		fileMenu.add(fileSaveAsItem);
		fileSaveAsItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				fileSaveAs();
			}
		});

		JMenuItem filePNGItem = new JMenuItem("Export Image",KeyEvent.VK_E);
		filePNGItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.ALT_MASK));
		fileMenu.add(filePNGItem);
		filePNGItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				filePNG();
			}
		});
				
		JMenuItem fileExitItem = new JMenuItem("Exit",KeyEvent.VK_X);
		fileExitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.ALT_MASK));
		fileMenu.add(fileExitItem);

		fileExitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				fileExit();
			}
		});
		
		
		// View Menu
		
		JMenu viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);
		menuBar.add(viewMenu);
		
		viewShowDescCheckBoxItem = new JCheckBoxMenuItem("Show Layout Description and Validity");
		viewShowDescCheckBoxItem.setMnemonic(KeyEvent.VK_D);
		viewShowDescCheckBoxItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.ALT_MASK));
		viewMenu.add(viewShowDescCheckBoxItem);
		
		viewShowDescCheckBoxItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setShowDesc ();
			}
		});
		
		viewShowStopwatchCheckBoxItem = new JCheckBoxMenuItem("Show Stopwatch");
		viewShowStopwatchCheckBoxItem.setMnemonic(KeyEvent.VK_T);
		viewShowStopwatchCheckBoxItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.ALT_MASK));
		viewMenu.add(viewShowStopwatchCheckBoxItem);
		
		viewShowStopwatchCheckBoxItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setShowStopwatch ();
			}
		});
		
		viewMenu.addSeparator();
		
		viewInitialLayoutItem = new JMenuItem("View Initial Layout",KeyEvent.VK_I);
		viewInitialLayoutItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, ActionEvent.ALT_MASK));
		viewInitialLayoutItem.setDisplayedMnemonicIndex(5);
		viewMenu.add(viewInitialLayoutItem);
		viewInitialLayoutItem.setEnabled(false);
		
		viewInitialLayoutItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				viewInitialLayout();
			}
		});
		
	    viewFinalLayoutItem = new JMenuItem("View Final Layout",KeyEvent.VK_F);
		viewFinalLayoutItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.ALT_MASK));
		viewMenu.add(viewFinalLayoutItem);
		viewFinalLayoutItem.setEnabled(false);
		
		viewFinalLayoutItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				viewFinalLayout();
			}
		});
		
		
		// Simple Utilities Menu
		JMenu utilitiesMenu = new JMenu("Utilities");
        utilitiesMenu.setMnemonic(KeyEvent.VK_U);
		menuBar.add(utilitiesMenu);
		
		JMenu utilitiesLoadRandomWFdiagMenu = new JMenu("Load a Random Wellformed Euler Diagram");
		utilitiesLoadRandomWFdiagMenu.setMnemonic(KeyEvent.VK_R);
		utilitiesMenu.add(utilitiesLoadRandomWFdiagMenu);
		
		ArrayList<DiagramUtility> diagramUtilityList = simpleDiagramPanel.getDiagramUtilityList();
		for(DiagramUtility du : diagramUtilityList) {
	        JMenuItem menuItem = new JMenuItem(du.getMenuText(), du.getMnemonicKey());
			menuItem.setAccelerator(KeyStroke.getKeyStroke(du.getAcceleratorKey(),0));
			menuItem.addActionListener(this);
			utilitiesLoadRandomWFdiagMenu.add(menuItem);
		}

		// Layout Menu
		JMenu layoutMenu = new JMenu("Layout");
        layoutMenu.setMnemonic(KeyEvent.VK_Y);
		menuBar.add(layoutMenu);
		
		ArrayList<SimpleDiagramDrawer> simpleDiagramDrawerList = simpleDiagramPanel.getSimpleDiagramDrawerList();
		for(SimpleDiagramDrawer sdd : simpleDiagramDrawerList) {
	        JMenuItem menuItem = new JMenuItem(sdd.getMenuText(), sdd.getMnemonicKey());
			menuItem.setAccelerator(KeyStroke.getKeyStroke(sdd.getAcceleratorKey(),0));
			menuItem.addActionListener(this);
			layoutMenu.add(menuItem);
		}
		
		
		this.addWindowListener(new java.awt.event.WindowAdapter() {
		    public void windowClosing(WindowEvent winEvt) {
		    	
		    	if (tempDir.exists()){
		    		String[] filesInDir = tempDir.list();
		    		if (filesInDir != null){
		    			for (int i=0; i<filesInDir.length; i++) {
                			File file = new File (tempDir.getAbsoluteFile()+"\\"+filesInDir[i]);
                			file.delete();
                		}
		    		}
		    	}
                tempDir.delete();
		    	
		        System.exit(0); 
		    }
		});
		
	}


	protected void fileNew() {
		if (currentFile != null) {
			if (!currentFile.isDirectory()) {
				currentFile = currentFile.getParentFile();
			}
		}
		simpleDiagramPanel.getSimpleConcreteDiagram().clear();
		viewInitialLayoutItem.setEnabled(false);
		viewFinalLayoutItem.setEnabled(false);
		simpleDiagramPanel.updateDisplay();
		
		initialSimpleConcreteDiagram = null;
		finalSimpleConcreteDiagram = null;
	}
	
	
	protected void fileOpen() {
		
		JFileChooser chooser = new JFileChooser(currentUserDir);
		chooser.setAcceptAllFileFilterUsed(false);
		FileFilter fileFilterEDP = new FileNameExtensionFilter("Euler Diagram with Polygons (*.edp)", "edp");
		chooser.setFileFilter(fileFilterEDP);
		
					
		int returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			currentFile = chooser.getSelectedFile();
			currentUserDir = chooser.getSelectedFile().getParent().toString();
			
			simpleDiagramPanel.getSimpleConcreteDiagram().clear();
			if (!simpleDiagramPanel.getSimpleConcreteDiagram().loadDiagram(currentFile)){
				
				JLabel label = new JLabel();
			    Font font = label.getFont();
				
			    StringBuffer style = new StringBuffer("font-family:" + font.getFamily() + ";");
			    style.append("font-weight:" + (font.isBold() ? "bold" : "normal") + ";");
			    style.append("font-size:" + font.getSize() + "pt;");
				
				Toolkit.getDefaultToolkit().beep();
					
			    JEditorPane ep = new JEditorPane("text/html", "<html><body style=\"" + style + "\">" + 
			             "Failed to load the diagram in " + currentFile.getAbsolutePath().toString()+
						 "<br><br> Check:"+
	                     "<br><br>&nbsp;1) The structure of the file" +
	                     	 "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;a) the properties of each polygon are defined in one line as" +
	                         "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;label | " +
	                         "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;followed by" +
	                         "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>x</i>-coordinate of vertex <i>i</i> | <i>y</i>-coordinate of vertex <i>i</i> |"+
	                         "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;for each vertex <i>i</i> of the polygon" +
	                         "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;b) one or more polygons are defined " +
	                         "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;c) the polygons are labelled a, b, c, ..." +
						 "<br><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;An example of a valid .edp file is available at "+
			                 "<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"http://www.eulerdiagrams.org/eulerForce/diagram_example.php\">http://www.eulerdiagrams.org/eulerForce/diagram_example.php</a>" +
			             "<br><br>&nbsp;2) The numbers defining the vertices of the polygons are rounded" +
			             "	  <br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;to the nearest whole number" +
						 "</body></html>");

			    ep.addHyperlinkListener(new HyperlinkListener()
			    {
					@Override
					public void hyperlinkUpdate(HyperlinkEvent e) {
						if (e.getEventType().equals(HyperlinkEvent.EventType.ACTIVATED)){	
							if (Desktop.isDesktopSupported()) {
						      try {
						        try {
									Desktop.getDesktop().browse(new URI(e.getURL().toString()));
								} catch (URISyntaxException e1) {
									e1.printStackTrace();
								}
						      } catch (IOException ex) {  }
						    } else {}
						}
					}
			    });
			    ep.setEditable(false);
			    ep.setBackground(label.getBackground());

			    JOptionPane.showMessageDialog(this, ep, "Opening a Diagram File", JOptionPane.ERROR_MESSAGE);
			}
			
			SimpleDiagramDrawerForceModel.centreOnScreen(simpleDiagramPanel.simpleConcreteDiagram.concreteContours, width, height);
			simpleDiagramPanel.getStopWatch().reset();
			simpleDiagramPanel.setShowStopWatch(showStopwatch);
			simpleDiagramPanel.setShowDiagramDetails(showDesc);
			simpleDiagramPanel.recordAsInitialLayout();
			viewInitialLayoutItem.setEnabled(false); 
			viewFinalLayoutItem.setEnabled(false);
			simpleDiagramPanel.updateDisplay();
			
			initialSimpleConcreteDiagram = null;
			finalSimpleConcreteDiagram = null;
			initialSimpleConcreteDiagram = simpleDiagramPanel.getSimpleConcreteDiagram().clone();
			
		}
	}

	
	protected void fileSaveAs() {
		
		JFileChooser chooser = new JFileChooser(currentUserDir);
		chooser.setAcceptAllFileFilterUsed(false);
		FileFilter fileFilterEDP = new FileNameExtensionFilter("Euler Diagram with Polygons (*.edp)", "edp");
		chooser.setFileFilter(fileFilterEDP);

		int returnVal = chooser.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			currentFile = chooser.getSelectedFile();
			File scdFile;
			if (currentFile.getAbsolutePath().endsWith(".edp")){
				scdFile = new File(currentFile.getAbsolutePath());
			} else{
				scdFile = new File(currentFile.getAbsolutePath()+".edp");
			}
			chooser = new JFileChooser(scdFile);
			simpleDiagramPanel.getSimpleConcreteDiagram().saveToFile(scdFile);
		}
	}
	
	
	protected void filePNG() {

		JFileChooser chooser = null;
		File pngFile = null;

		chooser = new JFileChooser(currentUserDir);
		chooser.setAcceptAllFileFilterUsed(false);
		FileFilter fileFilterEDP = new FileNameExtensionFilter("PNG (*.png)", "png");
		chooser.setFileFilter(fileFilterEDP);
		
		int returnVal = chooser.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			currentFile = chooser.getSelectedFile();
			pngFile = new File(currentFile.getAbsolutePath()+".png");
			chooser = new JFileChooser(pngFile);
			if (!currentFile.isDirectory()) {
				chooser.setSelectedFile(currentFile);
			}
		}

		try {
			BufferedImage image = new BufferedImage(getWidth(), getHeight(),BufferedImage.TYPE_INT_RGB);
			paint(image.getGraphics());
			ImageIO.write(image,"png",pngFile);
		} catch (Exception e) {}
		return;
	}

	
	protected void fileExit() {
		System.exit(0);
	}
	
	
	private void viewInitialLayout(){
		simpleDiagramPanel.getSimpleConcreteDiagram().clear();
		simpleDiagramPanel.getSimpleConcreteDiagram().concreteContours=initialSimpleConcreteDiagram.getConcreteContours();
		simpleDiagramPanel.updateDisplay();
	}
	
	
	private void viewFinalLayout(){
		simpleDiagramPanel.getSimpleConcreteDiagram().clear();
		simpleDiagramPanel.getSimpleConcreteDiagram().concreteContours=finalSimpleConcreteDiagram.getConcreteContours();
		simpleDiagramPanel.updateDisplay();
	}

	
	private void utilitiesLoadRandomWFdiagram (int noOfCurves){
		
		String diagLibDirPath = "WellformedEulerDiagLibrary/"+ noOfCurves + "curves";
			
		ArrayList<String> diagFilesList = new ArrayList<String>();
		String prefixInFilePath = ""; 
			
		
		File jarFile = new File(getClass().getProtectionDomain().getCodeSource().getLocation().getPath());

		if(jarFile.isFile()) {  // Run with JAR file
			prefixInFilePath = "";
			try {
				JarFile jar = new JarFile(jarFile);
			    Enumeration<JarEntry> entries = jar.entries(); 
			    while(entries.hasMoreElements()) {
			        String name = entries.nextElement().getName();
			        if (name.startsWith(diagLibDirPath + "/") && (name.endsWith(".edp"))) {
			        	diagFilesList.add(name);
			        }
			    }
			    jar.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else { // Run with IDE
			prefixInFilePath = diagLibDirPath + "/";
			URL url = Launcher.class.getResource("/" + diagLibDirPath);
		    if (url != null) {
		        try {
		            File apps = new File(url.toURI());
		            for (File app : apps.listFiles()) {
		            	String fileName = app.getName();
		            	if (fileName.endsWith(".edp")){
		            		diagFilesList.add(fileName);
		            	}
		            }
		        } catch (URISyntaxException ex) {
		        }
		    }
		}
		
    	int randomIndex = Utilities.randomIntegerInRange(0, diagFilesList.size()-1);
    	
		BufferedReader randomDiagFile = new BufferedReader(new InputStreamReader(getClass().getClassLoader().getResourceAsStream(prefixInFilePath+diagFilesList.get(randomIndex))));
        
		simpleDiagramPanel.getSimpleConcreteDiagram().clear();
		simpleDiagramPanel.getSimpleConcreteDiagram().loadDiagram(randomDiagFile);
		
		SimpleDiagramDrawerForceModel.centreOnScreen(simpleDiagramPanel.simpleConcreteDiagram.concreteContours, width, height);
		simpleDiagramPanel.getStopWatch().reset();
		simpleDiagramPanel.setShowStopWatch(showStopwatch);
		simpleDiagramPanel.setShowDiagramDetails(showDesc);
		simpleDiagramPanel.recordAsInitialLayout();
		viewInitialLayoutItem.setEnabled(false); 
		viewFinalLayoutItem.setEnabled(false);
		simpleDiagramPanel.updateDisplay();
	}


	
	@Override
	public void actionPerformed(ActionEvent event) {
		JMenuItem source = (JMenuItem)(event.getSource());
		
		ArrayList<SimpleDiagramDrawer> simpleDiagramDrawerList = simpleDiagramPanel.getSimpleDiagramDrawerList();
		for(SimpleDiagramDrawer sdd : simpleDiagramDrawerList) {
			if (sdd.getMenuText().equals(source.getText())) {
				initialSimpleConcreteDiagram = null;
				finalSimpleConcreteDiagram = null;
				initialSimpleConcreteDiagram = simpleDiagramPanel.getSimpleConcreteDiagram().clone();
				SimpleDiagramDrawerForceModel.centreOnScreen(simpleDiagramPanel.simpleConcreteDiagram.concreteContours, width, height);
				sdd.layout();
				Toolkit.getDefaultToolkit().beep();
				showDesc=true;
				showStopwatch=true;
				viewShowDescCheckBoxItem.setSelected(showDesc);
				viewShowStopwatchCheckBoxItem.setSelected(showStopwatch);
				simpleDiagramPanel.setShowDiagramDetails(true);
				simpleDiagramPanel.setShowStopWatch(true);
				viewInitialLayoutItem.setEnabled(true);
				viewFinalLayoutItem.setEnabled(true);
				finalSimpleConcreteDiagram = simpleDiagramPanel.getSimpleConcreteDiagram().clone();
				simpleDiagramPanel.updateDisplay();
				return;
			}
		}
		
		ArrayList<DiagramUtility> diagramUtilityList = simpleDiagramPanel.getDiagramUtilityList();
		for(DiagramUtility du : diagramUtilityList) {
			if (du.getMenuText().equals(source.getText())) {
				DiagramPanel dp = new DiagramPanel();
				du.setDiagramPanel(dp);
				
				utilitiesLoadRandomWFdiagram(((DiagramUtilityRandomWellformedDiagram)du).getNumberOfSets());
				
				SimpleDiagramDrawerForceModel.centreOnScreen(simpleDiagramPanel.simpleConcreteDiagram.concreteContours, width, height);
				simpleDiagramPanel.getStopWatch().reset();
				simpleDiagramPanel.setShowStopWatch(showStopwatch);
				simpleDiagramPanel.setShowDiagramDetails(showDesc);
				simpleDiagramPanel.recordAsInitialLayout();
				viewInitialLayoutItem.setEnabled(false);
				viewFinalLayoutItem.setEnabled(false);
				initialSimpleConcreteDiagram = null;
				finalSimpleConcreteDiagram = null;
				initialSimpleConcreteDiagram = simpleDiagramPanel.getSimpleConcreteDiagram().clone();
				
				simpleDiagramPanel.updateDisplay();
				return;
			}
		}
	}
}
	

