/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;


/**
 * 
 * StopWatch 
 * 
 * 
 * This is a modified version of Goldberg's 2005 StopWatch.java licensed as
 *  
 * 		Copyright (c) 2005, Corey Goldberg
	
 *	    StopWatch.java is free software; you can redistribute it and/or modify
 *	    it under the terms of the GNU General Public License as published by
 *	    the Free Software Foundation; either version 2 of the License, or
 *	    (at your option) any later version.
 *   
 * available at http://www.goldb.org/stopwatchjava.html
 *
 *
 */



public class StopWatch {
    
    private long startTime = 0;
    private long stopTime = 0;
    private boolean running = false;

    
    public void start() {
    	this.reset();
        this.startTime = System.currentTimeMillis();
        this.running = true;
    }

    
    public void stop() {
        this.stopTime = System.currentTimeMillis();
        this.running = false;
    }
    
    public void reset() {
    	this.startTime = 0;
        this.stopTime = 0;
        this.running = false;
    }

    
    //elapsed time in milliseconds
    public long getElapsedTimeMilliSecs() {
        long elapsed;
        if (running) {
             elapsed = (System.currentTimeMillis() - startTime);
        }
        else {
            elapsed = (stopTime - startTime);
        }
        return elapsed;
    }
    
    
    //elapsed time in seconds
    public long getElapsedTimeSecs() {
        long elapsed;
        if (running) {
            elapsed = ((System.currentTimeMillis() - startTime) / 1000);
        }
        else {
            elapsed = ((stopTime - startTime) / 1000);
        }
        return elapsed;
    }
    
    
    //elapsed time as string illustrating hours, mins, secs 
    public String getElapsedTimeString() {   
    	return convertMilliSecsToTimeString(getElapsedTimeMilliSecs());
    }
   
    
    //convert from millisecs (type long) to string time format illustrating hours:mins:secs 
    private String convertMilliSecsToTimeString (long msecsTime){ 
    	// based on 
    	//     http://www.coderanch.com/t/378404/Java-General/java/Convert-milliseconds-time
    	
    	String milliseconds = Integer.toString((int)(msecsTime % 1000)); 
    	long time = msecsTime / 1000;   
    	String seconds = Integer.toString((int)(time % 60));  
    	String minutes = Integer.toString((int)((time % 3600) / 60));  
    	String hours = Integer.toString((int)(time / 3600));
    	
    	for (int i = 0; i < 2; i++) {
    		if (milliseconds.length() < 2) {  
    			milliseconds = "0" + milliseconds;  
    		} 
    		if (seconds.length() < 2) {  
    			seconds = "0" + seconds;  
    		}  
    		if (minutes.length() < 2) {  
    			minutes = "0" + minutes;  
    		}  
    		if (hours.length() < 2) {  
    			hours = "0" + hours;  
    		}  
    	}
    	return (hours + ":" + minutes + ":" + seconds + ":" + milliseconds); 
    }
}