/* 
 * 
 * eulerForce v0.9.0
 * 
 * 2013-11-25
 *
 * 
 * 
 * eulerForce -- Laying Out Euler Diagrams Using a Force-Directed Approach	    
 * 		http://www.eulerdiagrams.org/eulerForce
 * 
 * 
 * 		Copyright (C) 2013, Luana Micallef and Peter Rodgers. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerForce.
 * 			
 * 		eulerForce is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerForce is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerForce (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package forcedirected;

import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Random;

import pjr.graph.triangulator.PolygonTriangulator;


/**
 * 
 * General Utilities
 *
 */


public class Utilities {
	
	
	// Random Operators

	
	/** Generate random integers in a certain range. 
	 *    the idea and most of the code was obtained from http://www.javapractices.com/topic/TopicAction.do?Id=62*/
	public static Integer randomIntegerInRange (int range_firstInt, int range_lastInt){
		
		// check the given range
	    if (range_firstInt > range_lastInt) {
		      System.out.println("Utilities.randomIntegerInRange:  inappropriate range - the first integer in the range is greater than the last one");
		      return null;
	    }
	    
	    Random random = new Random();
	    
	    //get the range, casting to long to avoid overflow problems
	    long range = (long)range_lastInt - (long)range_firstInt + 1;
	    
	    // compute a fraction of the range, 0 <= frac < range
	    long fraction = (long)(range * random.nextDouble());
	    int randomInt =  (int)(fraction + range_firstInt);  
	    
	    return (new Integer (randomInt));
	}
		
	
	
	
	 // Geometric Operations
	
	
	/**
	 * Find middle triangle, then find biggest containing rectangle.
	 */
	public static Point findMiddlePointInsidePolygon(Polygon polygon) {
		
		Point2D.Double pCentroid = computePolygonCentroid(polygon);
		int targetCentreX = pjr.graph.Util.convertToInteger(pCentroid.getX());
		int targetCentreY = pjr.graph.Util.convertToInteger(pCentroid.getY());

		double lastX = Double.MAX_VALUE;
		double lastY = Double.MAX_VALUE;
		PolygonTriangulator bt = new PolygonTriangulator();
		for(int i = 0 ; i < polygon.npoints; i++) {
			double x = polygon.xpoints[i];
			double y = polygon.ypoints[i];
			if(x == lastX && y == lastY) {
				// sometimes points are duplicated
				continue;
			}
			bt.addPolyPoint(x, y);		
			lastX = x;
			lastY = y;
		}
		ArrayList<Point2D> tris = bt.getTris();
		double minDistance = Double.MAX_VALUE;
		Point triangleMiddle = new Point(0,0);

		if(tris!=null){
			for(int j = 0 ; j < tris.size(); j+=3){
				Point2D t1 = tris.get(j);
				Point2D t2 = tris.get(j+1);
				Point2D t3 = tris.get(j+2);
				
				Point2D.Double centroid = new Point2D.Double((t1.getX()+t2.getX()+t3.getX())/3,(t1.getY()+t2.getY()+t3.getY())/3);

				double distance = pjr.graph.Util.distance(targetCentreX, targetCentreY, centroid.getX(), centroid.getY());
				if(minDistance > distance) {
					triangleMiddle = new Point(pjr.graph.Util.convertToInteger(centroid.getX()),pjr.graph.Util.convertToInteger(centroid.getY()));
					minDistance = distance;
				}

			}
		}

		Rectangle r = findLargestRectangleInPolygon(polygon, triangleMiddle);
		double retXD = r.getCenterX();
		double retYD = r.getCenterY();
		Point ret = new Point(pjr.graph.Util.convertToInteger(retXD),pjr.graph.Util.convertToInteger(retYD));
		return ret;
	}
	
	
	
	/**
	 * Find a rectangle inside the polygon starting at the given point,
	 * the point must be inside the polygon.
	 */
	public static Rectangle findLargestRectangleInPolygon(Polygon polygon, Point point) {

		Area area = new Area(polygon);
		final int startGrid = 1*1;//SCALE;
		int topX = point.x;
		int topY = point.y;
		double width = 0.01; // teeny rectangle to start with
		double height = 0.01;

		int grid = startGrid;
		boolean finish = false;
		while(!finish) {
			boolean noMove = true;
			Rectangle2D.Double r;
			
			int tryTopX = topX-grid;
			r = new Rectangle2D.Double(tryTopX,topY,width,height);

			if(area.contains(r)) {
				topX = tryTopX;
				width+=grid;
				noMove = false;
			}
			
			int tryTopY = topY-grid;
			r = new Rectangle2D.Double(topX,tryTopY,width,height);
			if(area.contains(r)) {
				topY = tryTopY;
				height+=grid;
				noMove = false;
			}
			
			double tryWidth = width+grid;
			r = new Rectangle2D.Double(topX,topY,tryWidth,height);
			if(area.contains(r)) {
				width = tryWidth;
				noMove = false;
			}
			
			double tryHeight = height+grid;
			r = new Rectangle2D.Double(topX,topY,width,tryHeight);
			if(area.contains(r)) {
				height = tryHeight;
				noMove = false;
			}
			
			if(grid == 1 && noMove) {
				finish = true;
			} else if(noMove) {
				grid = grid/2;
			}

		}
		
		Rectangle ret = new Rectangle(topX,topY,pjr.graph.Util.convertToInteger(width),pjr.graph.Util.convertToInteger(height));
		return ret;
		
	}
	
	
	
	/**
	 * Compute the centroid of a polygon.
	 */
	public static Point2D.Double computePolygonCentroid(Polygon p) {
		double cx = 0.0;
		double cy = 0.0;

		for(int i = 0 ; i < p.npoints-1; i++) {
			double a = p.xpoints[i] * p.ypoints[i+1] - p.xpoints[i+1] * p.ypoints[i];
			cx += (p.xpoints[i] + p.xpoints[i+1]) * a;
			cy += (p.ypoints[i] + p.ypoints[i+1]) * a;
		}
		double a = p.xpoints[p.npoints-1] * p.ypoints[0] - p.xpoints[0] * p.ypoints[p.npoints-1];
		cx += (p.xpoints[p.npoints-1] + p.xpoints[0]) * a;
		cy += (p.ypoints[p.npoints-1] + p.ypoints[0]) * a;
		  
		double area = computePolygonArea(p);

		cx /= 6 * area;
		cy /= 6 * area;	
		
		return new Point2D.Double(cx, cy);
	}
	
	
	
	/**
	 * Compute the area of a polygon.
	 */
	public static double computePolygonArea (Polygon p) {

		double area = 0.0;
		for (int i = 0; i < p.npoints - 1; i++) {
			area += (p.xpoints[i] * p.ypoints[i+1]) - (p.xpoints[i+1] * p.ypoints[i]);
		}
		area += (p.xpoints[p.npoints-1] * p.ypoints[0]) - (p.xpoints[0] * p.ypoints[p.npoints-1]);  

		area *= 0.5;
		
		if(area<0) {
			area =-area;
		}

		return area;
	}
	
	
}
